@@ -92,7 +92,6 @@ bool gcpGOfficeApplication::EditDocument (GOGChemPaintComponent *gogcp)
 		win = new gcpGOfficeWindow (this, gogcp);
 		if (!win)
 			return false;
-		AddWindow (win);
 		AddDocument (win->GetDocument ());
 		return true;
 	}
@@ -102,3 +101,17 @@ bool gcpGOfficeApplication::EditDocument (GOGChemPaintComponent *gogcp)
 		return false;
 	}
 }
+
+void gcpGOfficeApplication::OnFileNew ()
+{
+	gchar tmp[32];
+	if (m_pActiveDoc && !m_pActiveDoc->GetView ()->PrepareUnselect ())
+		return;
+	g_snprintf (tmp, sizeof (tmp), _("Untitled %d"), m_NumWindow++);
+	gcpWindow *win = new gcpWindow (this);
+	m_Docs.push_back (win->GetDocument ());
+}
+
+void gcpGOfficeApplication::OnFileClose ()
+{
+}
