@@ -4,9 +4,7 @@
  * GChemPaint library
  * application.cc 
  *
- * Copyright (C) 2004-2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -30,7 +28,6 @@
 #include "electron.h"
 #include "text.h"
 #include "plugin.h"
-#include "filechooser.h"
 #include "mendeleiev.h"
 #include "mesomer.h"
 #include "mesomery.h"
@@ -39,7 +36,11 @@
 #include "reactant.h"
 #include "reaction-step.h"
 #include "reaction-arrow.h"
+#include "tools.h"
+#include "window.h"
 #include "zoomdlg.h"
+#include <gcu/filechooser.h>
+#include <goffice/utils/go-file.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <sys/stat.h>
 #include <gconf/gconf-client.h>
@@ -50,6 +51,8 @@
 #include <openbabel/mol.h>
 #include <openbabel/obconversion.h>
 
+bool new_ui;
+
 // Objects creation static methods
 static Object* CreateMolecule()
 {
@@ -111,13 +114,14 @@ map<string, string> gcpApplication::m_SupportedPixbufFormats;
 #undef PACKAGE
 #define PACKAGE "gchempaint-unstable" 
 
-gcpApplication::gcpApplication (): Application (PACKAGE, DATADIR)
+gcpApplication::gcpApplication (): Application ("GChemPaint", DATADIR, PACKAGE, "gchempaint")
 {
 	Apps.push_back(this);
 	m_CurZ = 6;
 	m_pActiveDoc = NULL;
 	m_pActiveTool = NULL;
 	m_pNode = NULL;
+	m_NumWindow = 1;
 	FontName = g_strdup("Bitstream Vera Serif 12");
 	CurDir = g_get_current_dir ();
 	GConfClient* cli = gconf_client_get_default ();
@@ -244,20 +248,36 @@ gcpApplication::gcpApplication (): Application (PACKAGE, DATADIR)
 		gcpPlugin::LoadPlugins();
 		m_bInit = true;
 	}
+	RadioActions = NULL;
+	m_entries = 0;
+	IconFactory = gtk_icon_factory_new ();
 	set<gcpPlugin*>::iterator i = Plugins.begin (), end = Plugins.end ();
 	while (i != end) (*i++)->Populate (this);
+	gtk_icon_factory_add_default (IconFactory);
+	g_object_unref (G_OBJECT (IconFactory));
 	XmlDoc = xmlNewDoc ((xmlChar const*) "1.0");
+	visible_windows = 0;
+	m_idle = 0;
+	m_SupportedMimeTypes.push_back ("application/x-gchempaint");
+	m_SupportedMimeTypes.push_back ("chemical/x-chemdraw");
+	m_SupportedMimeTypes.push_back ("chemical/x-cml");
+	m_SupportedMimeTypes.push_back ("chemical/x-mdl-molfile");
+	m_SupportedMimeTypes.push_back ("chemical/x-pdb");
 }
 
 gcpApplication::~gcpApplication()
 {
 	map<string, gcpTool*>::iterator tool = Tools.begin(), endtool = Tools.end();
-	for (; tool!= endtool; tool++) delete (*tool).second;
-	Tools.clear();
-	Apps.remove(this);
+	for (; tool!= endtool; tool++)
+		delete (*tool).second;
+	Tools.clear ();
+	Apps.remove (this);
 	g_free (FontName);
-	if (m_pNode) bonobo_ui_node_free (m_pNode);
-	if (XmlDoc) xmlFreeDoc (XmlDoc);
+	if (m_pNode)
+		bonobo_ui_node_free (m_pNode);
+	if (XmlDoc)
+		xmlFreeDoc (XmlDoc);
+	m_SupportedMimeTypes.clear ();
 }
 
 void gcpApplication::ActivateMenu(const string& menuname, bool activate)
@@ -299,7 +319,7 @@ void gcpApplication::SetStatusText(const char* text)
 
 void gcpApplication::OnSaveAs()
 {
-	new gcpFileChooser (this, true, m_pActiveDoc);
+	FileChooser (this, true, m_SupportedMimeTypes, m_pActiveDoc);
 }
 
 void gcpApplication::AddUI (const char* filename)
@@ -344,9 +364,10 @@ void gcpApplication::AddUI (const char* filename)
 	}
 }
 
-bool gcpApplication::FileProcess(const gchar* filename, int filetype, bool bSave, GtkWindow *window, gcpDocument *pDoc)
+bool gcpApplication::FileProcess(const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *Doc)
 {
 	const gchar* ext;
+	gcpDocument *pDoc = static_cast<gcpDocument*> (Doc);
 	if (!filename || !strlen(filename) || filename[strlen(filename) - 1] == '/')
 	{
 		GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
@@ -356,33 +377,33 @@ bool gcpApplication::FileProcess(const gchar* filename, int filetype, bool bSave
 		gtk_widget_destroy (message);
 		return true;
 	}
-	int n = strlen(filename), i = n - 1;
+	if (!mime_type)
+		mime_type = "application/x-gchempaint";
+	int filetype = (mime_type)? 0: gcp_ft_Normal;
+	int n = strlen (filename), i = n - 1;
 	while ((i > 0) && (filename[i] != '.') && (filename[i] != '/')) i--;
 	if (filename[i] == '/') i = 0;
-	ext = NULL;
-	if (!filetype) //All files: search for the extension
-	{
-		if (i > 0)
-		{
-			ext = filename + i;
-			if (!strcmp(ext, ".gchempaint")) filetype = gcp_ft_Normal;
-			else if (!strcmp(ext, ".cml")) filetype = gcp_ft_CML;
-			else if (!strcmp(ext, ".mol")) filetype = gcp_ft_SDF;
-			else if (!strcmp(ext, ".sdf")) filetype = gcp_ft_SDF;
-			else if (!strcmp(ext, ".sd")) filetype = gcp_ft_SDF;
-			else if (!strcmp(ext, ".pdb")) filetype = gcp_ft_PDB;
-			else if (!strcmp(ext, ".ent")) filetype = gcp_ft_PDB;
-			else if (!strcmp(ext, ".chm")) filetype = gcp_ft_ChemDraw;
-			else {
-				GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-															_("Sorry, format not supported!"));
-				gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
-				gtk_dialog_run (GTK_DIALOG (message));
-				gtk_widget_destroy (message);
-				return true;
-			}
+	ext = (i > 0)? filename + i: NULL;
+	if (!filetype) {
+		// Check mime type
+		if (!strcmp (mime_type, "application/x-gchempaint"))
+			filetype = gcp_ft_Normal;
+		else if (!strcmp (mime_type, "chemical/x-cml"))
+			filetype = gcp_ft_CML;
+		else if (!strcmp (mime_type, "chemical/x-mdl-molfile"))
+			filetype = gcp_ft_SDF;
+		else if (!strcmp (mime_type, "chemical/x-pdb"))
+			filetype = gcp_ft_PDB;
+		else if (!strcmp (mime_type, "chemical/x-chemdraw"))
+			filetype = gcp_ft_ChemDraw;
+		else {
+			GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
+														_("Sorry, format not supported!"));
+			gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
+			gtk_dialog_run (GTK_DIALOG (message));
+			gtk_widget_destroy (message);
+			return true;
 		}
-		else filetype = gcp_ft_Normal;
 	}
 	if (!ext)
 		switch (filetype)
@@ -653,7 +674,10 @@ void gcpApplication::OpenGcp(const gchar* filename, gcpDocument* pDoc)
 		setlocale(LC_NUMERIC, "C");
 		old_time_locale = g_strdup(setlocale(LC_TIME, NULL));
 		setlocale(LC_TIME, "C");
-		if (!pDoc) pDoc = new gcpDocument(this, true);
+		if (!pDoc) {
+			OnFileNew ();
+			pDoc = m_pActiveDoc;
+		}
 		pDoc->SetFileName(filename, ".gchempaint");
 		bool result = pDoc->Load(xml->children);
 		setlocale(LC_NUMERIC, old_num_locale);
@@ -662,10 +686,9 @@ void gcpApplication::OpenGcp(const gchar* filename, gcpDocument* pDoc)
 		g_free(old_time_locale);
 		if (!result)
 		{
-			delete pDoc;
+			OnFileClose ();
 			throw (int) 4;
 		}
-		AddDocument(pDoc);
 		if (!(info->permissions & (GNOME_VFS_PERM_USER_WRITE | GNOME_VFS_PERM_GROUP_WRITE)))
 			pDoc->SetReadOnly (true);
 		gnome_vfs_file_info_unref (info);
@@ -732,20 +755,6 @@ void gcpApplication::InitTools()
 			(*i).second->Activate ((*i).first == "Select");
 }
 
-/*void gcpApplication::OnHelp (char* tag)
-{
-	if (!HelpBrowser.size () || !HelpFilename.size ())
-		return;
-	char *argv[3] = {NULL, NULL, NULL};
-	argv[0] = (char*) HelpBrowser.c_str();
-	string path = HelpFilename;
-	if (tag)
-		path += string("#") + string (PACKAGE) + string ("-") + string (tag);
-	argv[1] = (char*) path.c_str ();
-	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		NULL, NULL, NULL, NULL);
-}*/
-
 void gcpApplication::OnBug ()
 {
 	if (!WebBrowser.size ())
@@ -884,3 +893,154 @@ void gcpApplication::Zoom (double zoom)
 			new gcpZoomDlg (this);
 	}
 }
+
+static void on_tool_changed (GtkAction *action, GtkAction *current, gcpApplication* App)
+{
+	App->OnToolChanged (current);
+}
+
+void gcpApplication::OnToolChanged (GtkAction *current)
+{
+	if (m_pActiveTool)
+		m_pActiveTool->Activate(false);
+	m_pActiveTool = Tools[gtk_action_get_name (current)];
+	gcpTools *Tools = dynamic_cast<gcpTools*> (Dialogs["Tools"]);
+	if (Tools)
+		Tools->OnSelectTool (m_pActiveTool);		
+	if (m_pActiveTool)
+		m_pActiveTool->Activate(true);
+}
+
+void gcpApplication::BuildTools ()
+{
+	gcpTools *ToolsBox = new gcpTools (this);
+	map<int, string>::iterator i, iend = ToolbarNames.end ();
+	list<char const*>::iterator j, jend = UiDescs.end ();
+	string s;
+	GError *error = NULL;
+	GtkUIManager *ToolsManager = gtk_ui_manager_new ();
+	ToolsBox->SetUIManager (ToolsManager);
+	GtkActionGroup *action_group = gtk_action_group_new ("Tools");
+	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
+	gtk_action_group_add_radio_actions (action_group, RadioActions, m_entries, 0, G_CALLBACK (on_tool_changed), this);
+	gtk_ui_manager_insert_action_group (ToolsManager, action_group, 0);
+	for (j = UiDescs.begin (); j != jend; j++)
+		if (!gtk_ui_manager_add_ui_from_string (ToolsManager, *j, -1, &error)) {
+			g_message ("building user interface failed: %s", error->message);
+			g_error_free (error);
+			exit (EXIT_FAILURE);
+		}
+	for (i = ToolbarNames.begin (); i != iend; i++) {
+		s = "ui/";
+		s += (*i).second;
+		ToolsBox->AddToolbar (s);
+	}
+	g_object_unref (ToolsManager);
+	m_pActiveTool = Tools["Select"];
+	if (m_pActiveTool)
+		m_pActiveTool->Activate(true);
+	ToolsBox->OnSelectTool (m_pActiveTool);		
+	gcpMendeleievDlg::SetElement (this, m_CurZ);
+}
+
+void gcpApplication::ShowTools (bool visible)
+{
+	gcpTools *Tools = dynamic_cast<gcpTools*> (Dialogs["Tools"]);
+	if (!Tools) {
+		if (visible)
+			BuildTools ();
+	} else
+	Tools->Show (visible);
+}
+
+void gcpApplication::AddActions (GtkRadioActionEntry const *entries, int nb, char const *ui_description, gcpIconDesc const *icons)
+{
+	static int cur_entry = 1;
+	if (nb > 0) {
+		if (m_entries)
+			RadioActions = g_renew (GtkRadioActionEntry, RadioActions, m_entries + nb);
+		else
+			RadioActions = g_new (GtkRadioActionEntry, nb);
+		memcpy (RadioActions + m_entries, entries, nb * sizeof (GtkRadioActionEntry));
+		for (int i = 0; i < nb; i++)
+			if (strcmp (RadioActions[i + m_entries].name, "Select"))
+				RadioActions[i + m_entries].value = cur_entry++;
+			else
+				RadioActions[i + m_entries].value = 0;
+		m_entries += nb;
+	}
+	if (ui_description)
+		UiDescs.push_back (ui_description);
+	if (icons) {
+		GtkIconSet *set;
+		GtkIconSource *src;
+		while (icons->name) {
+			set = gtk_icon_set_new ();
+			src = gtk_icon_source_new ();
+		
+			gtk_icon_source_set_size_wildcarded (src, TRUE);
+			gtk_icon_source_set_pixbuf (src,
+				gdk_pixbuf_new_from_inline (-1, icons->data_24, FALSE, NULL));
+			gtk_icon_set_add_source (set, src);	/* copies the src */
+		
+			gtk_icon_factory_add (IconFactory, icons->name, set);	/* keeps reference to set */
+			gtk_icon_set_unref (set);
+			gtk_icon_source_free (src);
+			icons++;
+		}
+	}
+}
+
+void gcpApplication::RegisterToolbar (char const *name, int index)
+{
+	if (ToolbarNames[index] == "")
+		ToolbarNames[index] = name;
+}
+
+void gcpApplication::AddWindow (gcpWindow *window)
+{
+	m_Windows.insert (window);
+	NotifyIconification (false);
+}
+
+void gcpApplication::DeleteWindow (gcpWindow *window)
+{
+	m_Windows.erase (window);
+	ShowTools (false);
+}
+
+void gcpApplication::NotifyIconification (bool iconified)
+{
+	if (iconified) {
+		ShowTools (false);
+	}
+}
+
+static bool on_wait_focus (gcpApplication *App)
+{
+	App->CheckFocus ();
+	return false;
+}
+
+void gcpApplication::CheckFocus ()
+{
+	ShowTools (focused || gtk_grab_get_current ());
+	m_idle = 0;
+}
+
+void gcpApplication::NotifyFocus (bool has_focus, gcpWindow *window)
+{
+	if (window) {
+		m_pActiveWin = window;
+		m_pActiveDoc = window->GetDocument ();
+	}
+	focused = has_focus;
+	if (!m_idle)
+		g_idle_add ((GSourceFunc) on_wait_focus, this);
+}
+
+void gcpApplication::CloseAll ()
+{
+	while (!m_Windows.empty ())
+		(*m_Windows.begin ())->Destroy ();
+}
