@@ -72,7 +72,7 @@ public:
 	void SetCurZ(int Z) {m_CurZ = Z;}
 	int GetCurZ() {return m_CurZ;}
 	void OnSaveAs();
-	bool FileProcess(const gchar* filename, int filetype, bool bSave, GtkWindow *window, gcpDocument *pDoc = NULL);
+	bool FileProcess(const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *pDoc = NULL);
 	void SaveWithBabel(const gchar* filename, const gchar* ext, gcpDocument* pDoc, int Type);
 	void OpenWithBabel(const gchar* filename, const gchar* ext, gcpDocument* pDoc, int Type);
 	void SaveGcp(const gchar* filename, gcpDocument* pDoc);
@@ -102,6 +102,14 @@ public:
 	void AddWindow (gcpWindow *window);
 	void DeleteWindow (gcpWindow *window);
 	void NotifyIconification (bool iconified);
+	void NotifyFocus (bool has_focus, gcpWindow *window = NULL);
+	void CheckFocus ();
+	void CloseAll ();
+	list<char const*> &GetSupportedMimeTypes () {return m_SupportedMimeTypes;}
+
+	// virtual menus actions:
+	virtual void OnFileNew () = 0;
+	virtual void OnFileClose () = 0;
 
 protected:
 	virtual void AddDocument(gcpDocument* pDoc) = 0;
@@ -114,7 +122,8 @@ private:
 
 protected:
 	int m_CurZ;
-	gcpDocument* m_pActiveDoc;
+	gcpDocument *m_pActiveDoc;
+	gcpWindow *m_pActiveWin;
 	std::list<gcpDocument*> m_Docs;
 	map <string, GtkWidget*> Menus;
 	map <string, GtkWidget*> ToolItems;
@@ -131,6 +140,7 @@ protected:
 	string MailAgent;
 	xmlDocPtr XmlDoc;
 	char *CurDir;
+	unsigned m_NumWindow; //used for new files (Untitled%d)
 
 private:
 	GtkIconFactory *IconFactory;
@@ -138,10 +148,12 @@ private:
 	GtkRadioActionEntry* RadioActions;
 	int m_entries;
 	map<int, string> ToolbarNames;
-	unsigned m_NumWindow; //used for new files (Untitled%d)
 	unsigned m_NumDoc; //used to build the name of the action associated with the menu
 	std::set<gcpWindow*> m_Windows;
 	int visible_windows;
+	bool focused;
+	list<char const*> m_SupportedMimeTypes;
+	unsigned m_idle;
 };
 
 extern list<gcpApplication*> Apps;
