@@ -4,9 +4,7 @@
  * GChemPaint library
  * atom.cc
  *
- * Copyright (C) 2001-2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2005 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -43,6 +41,8 @@
 
 using namespace gcu;
 
+#define ATOM_EPSILON 0.1
+
 extern GtkTextTagTable* TextTagTable;
 
 gcpAtom::gcpAtom(): Atom()
@@ -197,18 +197,21 @@ bool gcpAtom::GetBestSide()
 	else return Element::BestSide(m_Z);
 }
 
-void gcpAtom::Update()
+void gcpAtom::Update ()
 {
-	if (m_ChargeAuto)
-	{
+	if (m_ChargeAuto) {
 		m_Charge = 0;
 		m_ChargeAuto = false;
 	}
+	if (m_ChargeAutoPos) {
+		NotifyPositionOccupation (m_ChargePos, false);
+		m_ChargePos = 0xff;
+	}
 	int nb, nexplp = 0, nexplu = 0; //nexplp is the number of explicit lone pairs
 	//nexplu is the number of explicit unpaired electrons
 	map<string, Object*>::iterator i;
 	gcpElectron* electron = (gcpElectron*) GetFirstChild (i);
-	while (electron){ 
+	while (electron) { 
 		if (electron->IsPair ())
 			nexplp++;
 		else
@@ -799,6 +802,9 @@ void gcpAtom::Update(GtkWidget* w)
 		if (item)
 			gtk_object_destroy (GTK_OBJECT (item));
 			g_object_set_data ((GObject*)group, "charge", NULL);
+			g_object_set_data ((GObject*)group, "figure", NULL);
+			g_object_set_data ((GObject*)group, "circle", NULL);
+			g_object_set_data ((GObject*)group, "sign", NULL);
 	}
 	map<string, Object*>::iterator i;
 	Object* electron = GetFirstChild (i);
@@ -840,14 +846,30 @@ void gcpAtom::UpdateAvailablePositions ()
 		while ((n != m_AngleList.end()) && (*n < angle)) n++;
 		m_AngleList.insert(n, angle);
 		i++;
-		if ((m_AvailPos & CHARGE_SW) && (angle >= 180.0) && (angle <= 270.0)) m_AvailPos -= CHARGE_SW;
-		if ((m_AvailPos & CHARGE_SE) && (((angle >= 270.0) && (angle <= 360.0)) || (fabs(angle) < 0.1))) m_AvailPos -= CHARGE_SE;
-		if ((m_AvailPos & CHARGE_S) && (angle >= 225.0) && (angle <= 315.0)) m_AvailPos -= CHARGE_S;
-		if ((m_AvailPos & CHARGE_NW) && (angle >= 90.0) && (angle <= 180.0)) m_AvailPos -= CHARGE_NW;
-		if ((m_AvailPos & CHARGE_NE) && (((angle >= 0.0) && (angle <= 90.0)) || (fabs(angle - 360.0) < 0.1))) m_AvailPos -= CHARGE_NE;
-		if ((m_AvailPos & CHARGE_N) && (angle >= 45.0) && (angle <= 135.0)) m_AvailPos -= CHARGE_N;
-		if ((m_AvailPos & CHARGE_W) && ((angle <= 225.0) && (angle >= 135.0))) m_AvailPos -= CHARGE_W;
-		if ((m_AvailPos & CHARGE_E) && ((angle >= 315.0) || (angle <= 45.0))) m_AvailPos -= CHARGE_E;
+		if ((m_AvailPos & CHARGE_SW) && (angle >= 180.0 - ATOM_EPSILON) &&
+			(angle <= 270.0 + ATOM_EPSILON))
+			m_AvailPos -= CHARGE_SW;
+		if ((m_AvailPos & CHARGE_SE) && (((angle >= 270.0 - ATOM_EPSILON) &&
+			(angle <= 360.0 + ATOM_EPSILON)) || (fabs(angle) < ATOM_EPSILON)))
+			m_AvailPos -= CHARGE_SE;
+		if ((m_AvailPos & CHARGE_S) && (angle >= 225.0 - ATOM_EPSILON) &&
+			(angle <= 315.0 + ATOM_EPSILON))
+			m_AvailPos -= CHARGE_S;
+		if ((m_AvailPos & CHARGE_NW) && (angle >= 90.0 - ATOM_EPSILON) &&
+			(angle <= 180.0 + ATOM_EPSILON))
+			m_AvailPos -= CHARGE_NW;
+		if ((m_AvailPos & CHARGE_NE) && (((angle >= 0.0 - ATOM_EPSILON) &&
+			(angle <= 90.0 + ATOM_EPSILON)) || (fabs(angle - 360.0) < ATOM_EPSILON)))
+			m_AvailPos -= CHARGE_NE;
+		if ((m_AvailPos & CHARGE_N) && (angle >= 45.0 - ATOM_EPSILON) &&
+			(angle <= 135.0 + ATOM_EPSILON))
+			m_AvailPos -= CHARGE_N;
+		if ((m_AvailPos & CHARGE_W) && ((angle <= 225.0 + ATOM_EPSILON) &&
+			(angle >= 135.0 - ATOM_EPSILON)))
+			m_AvailPos -= CHARGE_W;
+		if ((m_AvailPos & CHARGE_E) && ((angle >= 315.0 - ATOM_EPSILON) ||
+			(angle <= 45.0 + ATOM_EPSILON)))
+			m_AvailPos -= CHARGE_E;
 	}
 	m_AngleList.push_back ((angle = m_AngleList.front ()) + 360.0);
 	m_InterBonds.clear ();
@@ -1562,3 +1584,30 @@ void gcpAtom::SetCharge (int charge)
 	m_ChargeAuto = false;
 	Update ();
 }
+
+void gcpAtom::Transform2D (Matrix2D& m, double x, double y)
+{
+	Atom::Transform2D (m, x, y);
+	// Now transform electrons
+	map<string, Object*>::iterator i;
+	Object* electron = GetFirstChild (i);
+	while (electron) {
+		electron->Transform2D (m, x, y);
+		electron = GetNextChild (i);
+	}
+	if (GetCharge ()) {
+		if (m_ChargeAutoPos) {
+			if (m_ChargePos > 0)
+				NotifyPositionOccupation (m_ChargePos, false);
+			m_ChargePos = 0xff;
+			Update ();
+		} else {
+			double xc = cos (m_ChargeAngle), yc = - sin (m_ChargeAngle);
+			m.Transform (xc, yc);
+			m_ChargeAngle = atan2 (- yc, xc);
+			if (m_ChargeAngle < 0)
+				m_ChargeAngle += 2 * M_PI;
+			SetChargePosition (0, FALSE, m_ChargeAngle, m_ChargeDist);
+		}
+	}
+}
