@@ -33,6 +33,7 @@
 #include "reaction.h"
 #include "mesomery.h"
 #include "text.h"
+#include "window.h"
 #include <locale.h>
 #include <unistd.h>
 #include <libgen.h>
@@ -56,9 +57,9 @@ gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone, gcpWindow *window
 	m_filename = NULL;
 	m_label = NULL;
 	m_title = NULL;
+	m_bWriteable = true;
 	m_pView = new gcpView(this, !StandAlone);
 	m_bDirty = m_bIsLoading = m_bUndoRedo = false;
-	m_bWriteable = true;
 	m_FileType = gcp_ft_Normal;
 	m_DocPropDlg = NULL;
 	g_date_set_time(&CreationDate, time(NULL));
@@ -81,6 +82,8 @@ gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone, gcpWindow *window
 	m_pCurOp = NULL;
 	m_bReadOnly = false;
 	SetActive ();
+	if (window && App)
+		App->AddWindow (window);
 }
 
 gcpDocument::~gcpDocument ()
@@ -110,7 +113,8 @@ gcpDocument::~gcpDocument ()
 		obj->Lock ();
 		Remove (obj);
 	}
-	m_pApp->RemoveDocument (this);
+	if (m_pApp)
+		m_pApp->RemoveDocument (this);
 	if (m_pView)
 		delete m_pView;
 }
@@ -176,8 +180,8 @@ bool gcpDocument::ImportOB(OBMol& Mol)
 	OBBond *bond;
 	vector<OBEdgeBase*>::iterator j;
 	bond = Mol.BeginBond(j);
-	double l = floor(DefaultBondLength / bond->GetLength());
-	if (l == 0) l = 1;
+	double l = (bond)? floor(DefaultBondLength / bond->GetLength()): 1.;
+	if (l == 0.) l = 1.;
 	m_title = g_strdup(Mol.GetTitle());
 	OBAtom *atom;
 	gcpAtom* pAtom;
@@ -572,6 +576,8 @@ bool gcpDocument::Load (xmlNodePtr root)
 			xmlFree (tmp);
 		}
 	}
+	if (m_Window)
+		m_Window->SetTitle (GetTitle ());
 	node = GetNodeByName (root, "author");
 	if (node)
 	{
@@ -633,7 +639,7 @@ xmlDocPtr gcpDocument::BuildXMLTree()
 	if (xml == NULL)
 		throw (int) 0;
 	
-	xml->children =  xmlNewDocNode (xml, NULL, (xmlChar*) "chemistry", NULL);
+	xmlDocSetRootElement (xml,  xmlNewDocNode (xml, NULL, (xmlChar*) "chemistry", NULL));
 	ns = xmlNewNs (xml->children, (xmlChar*) "http://www.nongnu.org/gchempaint", (xmlChar*) "gcp");
 	xmlSetNs (xml->children, ns);
 	if (!g_date_valid (&CreationDate))
@@ -994,11 +1000,13 @@ void gcpDocument::PushOperation(gcpOperation* operation, bool undo)
 
 void gcpDocument::SetActive ()
 {
-	m_pApp->ActivateMenu("Undo", !m_UndoList.empty ());
-	m_pApp->ActivateMenu("Redo", !m_RedoList.empty ());
-	m_pApp->ActivateMenu("Image", HasChildren ());
-	m_pApp->ActivateMenu ("Save", !m_bReadOnly);
-	m_pApp->ActivateToolItem ("Save", !m_bReadOnly);
+	if (m_pApp) {
+		m_pApp->ActivateMenu("Undo", !m_UndoList.empty ());
+		m_pApp->ActivateMenu("Redo", !m_RedoList.empty ());
+		m_pApp->ActivateMenu("Image", HasChildren ());
+		m_pApp->ActivateMenu ("Save", !m_bReadOnly);
+		m_pApp->ActivateToolItem ("Save", !m_bReadOnly);
+	}
 }
 
 extern xmlDocPtr pXmlDoc;
