@@ -4,9 +4,7 @@
  * GChemPaint library
  * mesomery.cc 
  *
- * Copyright (C) 2002-2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2005 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -131,7 +129,6 @@ typedef struct
 bool gcpMesomery::Build (list<Object*>& Children) throw (invalid_argument)
 {
 	gcpDocument *pDoc = reinterpret_cast<gcpDocument *> (GetDocument ());
-	gcpView *pView = pDoc->GetView ();
 	gcpWidgetData  *pData= reinterpret_cast<gcpWidgetData *> (g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data"));
 	list<Object *>::iterator i, iend = Children.end ();
 	map<Object *, ObjectData> Objects;
@@ -319,7 +316,7 @@ static void TerminateChain (MesomeryChain *chain, set<Object*> &core,
 
 static void DoAlign (gcpMesomeryArrow *arrow, MesomerData &start, MesomerData &end, double ArrowPadding, double ZoomFactor)
 {
-	double x0, y0, x1, y1, x2, y2, x, y, l, dx, dy;
+	double x0, y0, x1, y1, x, y, l, dx, dy;
 	bool horiz;
 	arrow->GetCoords (&x0, &y0, &x1, &y1);
 	x = x1 - x0;
@@ -390,7 +387,7 @@ static double GetProjectionRatio (MesomerData &md, MesomerData &md0, MesomerData
 
 static void AlignArrow (gcpMesomeryArrow *arrow, MesomerData &md0, MesomerData &md1, double ArrowPadding, double ZoomFactor)
 {
-	double d, dx, dy, x, y, x0, y0, x1, y1, l;
+	double dx, dy, x, y, x0, y0, x1, y1, l;
 	bool horiz;
 	if (arrow->GetStartMesomer () != md0.m)
 		arrow->Reverse ();
@@ -465,7 +462,7 @@ static void ChainAdjust (MesomerData &md0, MesomerData &md1, double ratio, map<O
 {
 	MesomeryChain *chain = md0.chain, *parent;
 	MesomerData md;
-	double x, y;
+	double x = 0., y = 0.;
 	ratio -= 1.;
 	list<gcpMesomer*>::reverse_iterator m, mend;
 	list<struct MC*>::iterator c, cend;
@@ -564,7 +561,6 @@ void gcpMesomery::Align ()
 	}
 	/* now add one mesomer to each growing chain and terminate chains when the end is
 	reached or when a cycle is found */
-	bool CreateChains;
 	while (!chains.empty ()) {
 		cend = chains.end ();
 		for (c = chains.begin (); c != cend; c++) {
