@@ -4,7 +4,7 @@
  * GChemPaint library
  * settings.cc 
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -25,6 +25,7 @@
 #include "gchempaint-config.h"
 #include "settings.h"
 double DefaultBondLength = 140;
+double DefaultBondAngle = 120;
 double DefaultBondDist = 20;
 double DefaultBondWidth = 4.0;
 double DefaultArrowLength = 200;
@@ -44,14 +45,16 @@ double DefaultSignPadding = 32.0;
 double ChargeSignHeight = 0.;
 double ChargeSignSize = 0.;
 double ChargeYAlign = 0.;
-gchar* Color = const_cast<char*> ("black");
-gchar* DeleteColor = const_cast<char*> ("red");
-gchar* AddColor = const_cast<char*> ("green");
-gchar* SelectColor = const_cast<char*> ("cyan");
-gchar* DefaultFontFamily = const_cast<char*> ("Bitstream Vera Sans");
+gchar *Color = "black";
+gchar *DeleteColor = "red";
+gchar *AddColor = "green";
+gchar *SelectColor = "cyan";
+gchar *DefaultFontFamily ="Bitstream Vera Sans";
+gchar *DefaultTextFontFamily ="Bitstream Vera Serif";
 PangoStyle DefaultFontStyle = PANGO_STYLE_NORMAL;
 PangoWeight DefaultFontWeight = PANGO_WEIGHT_NORMAL;
 PangoVariant DefaultFontVariant = PANGO_VARIANT_NORMAL;
 PangoStretch DefaultFontStretch = PANGO_STRETCH_NORMAL;
 gint DefaultFontSize = 12 * PANGO_SCALE;
 unsigned MaxStackSize = 0;//infinite size authorized for undo:redo stacks
+bool MergeAtoms = true;
