@@ -4,9 +4,7 @@
  * GChemPaint library
  * widgetdata.cc
  *
- * Copyright (C) 2002-2004
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2004 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -205,7 +203,7 @@ void gcpWidgetData::Copy(GtkClipboard* clipboard)
 	*pDoc = xmlNewDoc((xmlChar*)"1.0");
 	if (!*pDoc) return;
 	if (SelectedObjects.empty()) return;
-	(*pDoc)->children = xmlNewDocNode(*pDoc, NULL, (xmlChar*)"chemistry", NULL);
+	xmlDocSetRootElement (*pDoc, xmlNewDocNode(*pDoc, NULL, (xmlChar*)"chemistry", NULL));
 //FIXME: implement exception handling
 	std::list<Object*>::iterator i;
 	xmlNodePtr child;
@@ -224,7 +222,7 @@ void gcpWidgetData::GetObjectBounds(Object* obj, ArtDRect &rect)
 	if ((group = Items[obj]))
 	{
 		gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(group), &x1, &y1, &x2, &y2);
-		if (rect.x0 < 0)
+		if (rect.x0 < -9.)
 		{
 			rect.x0 = x1;
 			rect.y0 = y1;
@@ -253,14 +251,14 @@ void gcpWidgetData::GetObjectBounds(Object* obj, ArtDRect &rect)
 void gcpWidgetData::GetSelectionBounds(ArtDRect &rect)
 {
 	std::list<Object*>::iterator i, j;
-	rect.x0 = -1.;
+	rect.x0 = -10.;
 	for (i = SelectedObjects.begin(); i != SelectedObjects.end(); i++)
 		GetObjectBounds(*i, rect);
 }
 
 void gcpWidgetData::GetObjectBounds(Object* obj, ArtDRect *rect)
 {
-	rect->x0 = -1.;
+	rect->x0 = -10.;
 	GetObjectBounds(obj, *rect);
 }
 
