@@ -26,6 +26,11 @@
 #include "window.h"
 #include "about.h"
 #include "zoomdlg.h"
+#include <gcu/filechooser.h>
+#include <libgnomeprint/gnome-print.h>
+#include <libgnomeprint/gnome-print-job.h>
+#include <libgnomeprintui/gnome-print-dialog.h>
+#include <libgnomeprintui/gnome-print-job-preview.h>
 
 static void on_destroy (GtkWidget* widget, gcpWindow* Win)
 {
@@ -67,6 +72,16 @@ void on_print(GtkWidget* widget, gcpWindow* Win)
 	Win->OnPrint();
 }
 
+static void on_file_close (GtkWidget* widget, gcpWindow *Win)
+{
+	Win->Close ();
+}
+
+static void on_quit (GtkWidget* widget, gcpWindow *Win)
+{
+	Win->GetApplication ()->CloseAll ();
+}
+
 static void on_cut_selection(GtkWidget* widget, gcpWindow* Win)
 {
 	Win->OnCutSelection();
@@ -164,9 +179,17 @@ static void on_zoom (GtkWidget* widget, gcpWindow* Win)
 	Win->Zoom (0.);
 }
 
-static bool on_focus(GtkWidget *widget, GdkEventFocus *event, gcpWindow* Win)
+static bool on_focus_in (GtkWidget *widget, GdkEventFocus *event, gcpWindow* Win)
 {
-	gtk_clipboard_request_contents (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), gdk_atom_intern ("TARGETS", FALSE),  (GtkClipboardReceivedFunc) on_receive_targets, Win->GetApplication ());
+	gcpApplication *App = Win->GetApplication ();
+	App->NotifyFocus (true, Win);
+	gtk_clipboard_request_contents (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), gdk_atom_intern ("TARGETS", FALSE),  (GtkClipboardReceivedFunc) on_receive_targets, App);
+	return false;
+}
+
+static bool on_focus_out (GtkWidget *widget, GdkEventFocus *event, gcpWindow* Win)
+{
+	Win->GetApplication ()->NotifyFocus (false);
 	return false;
 }
 
@@ -265,6 +288,10 @@ static GtkActionEntry entries[] = {
 		  N_("Print the current file"), G_CALLBACK (on_print) },
 	  { "Properties", GTK_STOCK_PROPERTIES, N_("Prope_rties..."), NULL,
 		  N_("Modify the file's properties"), G_CALLBACK (on_properties) },
+	  { "Close", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
+		  N_("Close the current file"), G_CALLBACK (on_file_close) },
+	  { "Quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
+		  N_("Quit GChemPaint"), G_CALLBACK (on_quit) },
   { "EditMenu", NULL, N_("_Edit") },
 	  { "Undo", GTK_STOCK_UNDO, N_("_Undo"), "<control>Z",
 		  N_("Undo the last action"), G_CALLBACK (on_undo) },
@@ -332,6 +359,7 @@ static const char *ui_description =
 "      <separator name='file-sep2'/>"
 "      <menuitem action='Properties'/>"
 "      <separator name='file-sep3'/>"
+"      <menuitem action='Close'/>"
 "	   <placeholder name='file1'/>"
 "    </menu>"
 "    <menu action='EditMenu'>"
@@ -405,7 +433,7 @@ static const char *ui_web_description =
 "  </menubar>"
 "</ui>";
 
-gcpWindow::gcpWindow (gcpApplication *App)
+gcpWindow::gcpWindow (gcpApplication *App, char const *extra_ui)
 {
 	GtkWidget *vbox;
 	GtkWidget *bar;
@@ -417,11 +445,13 @@ gcpWindow::gcpWindow (gcpApplication *App)
 	m_Doc = new gcpDocument (App, true, this);
 	GtkWidget *w = m_Doc->GetView ()->CreateNewWidget ();
 	m_Window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
-	gtk_window_set_position (m_Window, GTK_WIN_POS_CENTER);
-	gtk_window_set_title (m_Window, _("GChemPaint"));
+	g_object_set_data (G_OBJECT (m_Window), "gcp-role", (void*) 1);
+	gtk_window_set_position (m_Window, GTK_WIN_POS_MOUSE);
+	gtk_window_set_title (m_Window, m_Doc->GetTitle ());
 	gtk_window_set_icon_name (m_Window, "gchempaint");
 	g_signal_connect (G_OBJECT (m_Window), "destroy", G_CALLBACK (on_destroy), this);
-	g_signal_connect (G_OBJECT (m_Window), "focus_in_event", G_CALLBACK (on_focus), this);
+	g_signal_connect (G_OBJECT (m_Window), "focus_in_event", G_CALLBACK (on_focus_in), this);
+	g_signal_connect (G_OBJECT (m_Window), "focus_out_event", G_CALLBACK (on_focus_out), this);
 	g_signal_connect (G_OBJECT (m_Window), "window-state-event", G_CALLBACK (on_state), this);
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (m_Window), vbox);
@@ -441,16 +471,23 @@ gcpWindow::gcpWindow (gcpApplication *App)
 	gtk_window_add_accel_group (GTK_WINDOW (m_Window), accel_group);
 	
 	error = NULL;
-	if (!gtk_ui_manager_add_ui_from_string (m_UIManager, ui_description, -1, &error))
-	  {
+	if (!gtk_ui_manager_add_ui_from_string (m_UIManager, ui_description, -1, &error)) {
 		g_message ("building menus failed: %s", error->message);
 		g_error_free (error);
 		exit (EXIT_FAILURE);
-	  }
-	if (App->HasWebBrowser ())
-		gtk_ui_manager_add_ui_from_string (m_UIManager, ui_web_description, -1, &error);
-	if (App->HasMailAgent ())
-		gtk_ui_manager_add_ui_from_string (m_UIManager, ui_mail_description, -1, &error);
+	}
+	if (App->HasWebBrowser () && !gtk_ui_manager_add_ui_from_string (m_UIManager, ui_web_description, -1, &error)) {
+		g_message ("building menus failed: %s", error->message);
+		g_error_free (error);
+	}
+	if (App->HasMailAgent () && !gtk_ui_manager_add_ui_from_string (m_UIManager, ui_mail_description, -1, &error)) {
+		g_message ("building menus failed: %s", error->message);
+		g_error_free (error);
+	}
+	if (extra_ui && !gtk_ui_manager_add_ui_from_string (m_UIManager, extra_ui, -1, &error)) {
+		g_message ("building menus failed: %s", error->message);
+		g_error_free (error);
+	}
 
 	bar = gtk_ui_manager_get_widget (m_UIManager, "/MainMenu");
 	gtk_box_pack_start (GTK_BOX (vbox), bar, false, false, 0);
@@ -467,11 +504,12 @@ gcpWindow::gcpWindow (gcpApplication *App)
 	m_Bar = gtk_statusbar_new ();
 	m_statusId = gtk_statusbar_get_context_id (GTK_STATUSBAR (m_Bar), "status");
 	gtk_statusbar_push (GTK_STATUSBAR (m_Bar), m_statusId, _("Ready"));
+	m_MessageId = 0;
 	gtk_box_pack_start (GTK_BOX (vbox), m_Bar, false, false, 0);
 		
 	g_signal_connect(GTK_OBJECT(m_Window), "key_press_event", (GCallback)on_key_press, this);
 	g_signal_connect(GTK_OBJECT(m_Window), "key_release_event", (GCallback)on_key_release, this);
-	
+
 	gtk_widget_show_all (GTK_WIDGET (m_Window));
 	App->SetActiveDocument (m_Doc);
 }
@@ -483,14 +521,20 @@ gcpWindow::~gcpWindow ()
 
 void gcpWindow::OnFileNew()
 {
+	m_App->OnFileNew ();
 }
 
 void gcpWindow::OnSave()
 {
+	if (m_Doc->GetFileName ())
+		m_Doc->Save();
+	else
+		m_App->OnSaveAs ();
 }
 
 void gcpWindow::OnFileOpen()
 {
+	FileChooser (m_App, false, m_App->GetSupportedMimeTypes (), (!m_Doc->HasChildren () && !m_Doc->IsDirty ())? m_Doc: NULL);
 }
 
 void gcpWindow::OnProperties()
@@ -500,6 +544,44 @@ void gcpWindow::OnProperties()
 
 void gcpWindow::OnPrint()
 {
+	GnomePrintConfig* config = gnome_print_config_default ();
+	GnomePrintContext *pc;// = gnome_print_context_new (config);
+	GnomePrintJob *gpj = gnome_print_job_new (config);
+	int do_preview = 0, copies = 1, collate = 0;
+	GnomePrintDialog *gpd;
+	gpd = GNOME_PRINT_DIALOG (gnome_print_dialog_new(gpj, (const guchar*)_("Print"), GNOME_PRINT_DIALOG_COPIES));
+	gtk_window_set_icon_name (GTK_WINDOW (gpd), "gchempaint");
+	gnome_print_dialog_set_copies (gpd, copies, collate);
+	switch (gtk_dialog_run (GTK_DIALOG (gpd))) {
+	case GNOME_PRINT_DIALOG_RESPONSE_PRINT:
+		do_preview = 0;
+		break;
+	case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
+		do_preview = 1;
+		break;
+	case GNOME_PRINT_DIALOG_RESPONSE_CANCEL:
+		gtk_widget_destroy (GTK_WIDGET (gpd));
+		return;
+	}
+	gtk_widget_destroy (GTK_WIDGET (gpd));
+	pc = gnome_print_job_get_context (gpj);
+	gnome_print_beginpage (pc, (const guchar*)"");
+	gdouble width, height;
+	gnome_print_config_get_double (config, (guchar const*) GNOME_PRINT_KEY_PAPER_WIDTH, &width);
+	gnome_print_config_get_double (config, (guchar const*) GNOME_PRINT_KEY_PAPER_HEIGHT, &height);
+	m_Doc->Print (pc, width, height);
+	gnome_print_showpage (pc);
+	g_object_unref (pc);
+	gnome_print_job_close (gpj);
+	if (do_preview) {
+		GtkWidget *preview = gnome_print_job_preview_new (gpj, (const guchar*)_("Preview"));
+		gtk_window_set_icon_name (GTK_WINDOW (preview), "gchempaint");
+		gtk_widget_show (preview);
+	} else {
+		gnome_print_job_print (gpj);
+	}
+	g_object_unref (gpj);
+	gnome_print_config_unref (config);
 }
 
 void gcpWindow::SetActive (gcpDocument* pDoc, GtkWidget* w)
@@ -581,7 +663,8 @@ void gcpWindow::ClearStatus()
 
 void gcpWindow::SetStatusText(const char* text)
 {
-	if (m_MessageId) gtk_statusbar_pop (GTK_STATUSBAR (m_Bar), m_statusId);
+	if (m_MessageId)
+		gtk_statusbar_pop (GTK_STATUSBAR (m_Bar), m_statusId);
 	m_MessageId = gtk_statusbar_push (GTK_STATUSBAR (m_Bar), m_statusId, text);
 }
 
@@ -613,3 +696,15 @@ bool gcpWindow::OnKeyReleased(GtkWidget* widget, GdkEventKey* ev)
 		m_Doc->GetView ()->OnKeyRelease (m_Doc->GetWidget (), ev):
 		false;
 }
+
+void gcpWindow::Close ()
+{
+	m_Doc->GetView ()->PrepareUnselect ();
+	gtk_widget_destroy (GTK_WIDGET (m_Window));
+}
+
+
+char const *gcpWindow::GetDefaultTitle ()
+{
+	return _("GChemPaint");
+}
