@@ -4,7 +4,7 @@
  * GChemPaint arrows plugin
  * arrowtool.cc 
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2005 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -34,10 +34,10 @@
 #include "lib/mesomery-arrow.h"
 #include "lib/reaction-arrow.h"
 #include "lib/settings.h"
-#include <cmath>
+#include <math.h>
 #include <gconf/gconf-client.h>
 
-static char const *ToolNames[] = {
+static char* ToolNames[] = {
 	"SimpleArrow",
 	"ReversibleArrow",
 	"ReversibleArrow",
@@ -46,27 +46,28 @@ static char const *ToolNames[] = {
 };
 
 
-class gcpArrowToolDlg: public gcpDialog
+class gcpArrowToolDlg: public Dialog
 {
 public:
 	gcpArrowToolDlg (gcpApplication* App, bool FullHeads);
 	virtual ~gcpArrowToolDlg ();
 
 	virtual bool Apply ();
-	void OnDefault ();
 	
 private:
 	GtkToggleButton *full_btn;
 };
 
-static void on_default (gcpArrowToolDlg* dlg)
+static void on_default (GtkToggleButton *button)
 {
-	dlg->OnDefault ();
+	GConfClient *conf_client = gconf_client_get_default ();
+	gconf_client_set_bool (conf_client, "/apps/gchempaint/plugins/arrows/full-arrows-heads", gtk_toggle_button_get_active (button), NULL);
+	g_object_unref (conf_client);
 }
 
-gcpArrowToolDlg::gcpArrowToolDlg (gcpApplication* App, bool FullHeads): gcpDialog(App, GLADEDIR"/arrowtool.glade", "arrowtool")
+gcpArrowToolDlg::gcpArrowToolDlg (gcpApplication* App, bool FullHeads): Dialog(App, DATADIR"/gchempaint-unstable/ui/arrowtool.glade", "arrowtool")
 {
-	GtkTable *table = GTK_TABLE (glade_xml_get_widget (xml, "heads_table"));
+	GtkTable *table = GTK_TABLE (glade_xml_get_widget (xml, "heads-table"));
 	GnomeCanvas *canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
 	double width = (DefaultArrowLength + 2 * DefaultArrowPadding) * DefaultZoomFactor,
 		height = (DefaultArrowDist + DefaultBondWidth + 2 * (DefaultArrowHeadB + DefaultPadding)) * DefaultZoomFactor;
@@ -144,31 +145,25 @@ gcpArrowToolDlg::gcpArrowToolDlg (gcpApplication* App, bool FullHeads): gcpDialo
 	full_btn = GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "full"));
 	if (FullHeads) gtk_toggle_button_set_active (full_btn, true);
 	GtkWidget* w = glade_xml_get_widget (xml, "default");
-	g_signal_connect_swapped (w, "clicked", G_CALLBACK (on_default), this);
-	App->SetDialog ("ArrowHeads", this);
+	g_signal_connect_swapped (w, "clicked", G_CALLBACK (on_default), full_btn);
+	reinterpret_cast<gcpApplication*> (App)->SetDialog ("ArrowHeads", this);
 	gtk_widget_show_all (GTK_WIDGET (dialog));
 }
 
 gcpArrowToolDlg::~gcpArrowToolDlg ()
 {
-	m_App->SetDialog ("ArrowHeads", NULL);
+	reinterpret_cast<gcpApplication*> (m_App)->SetDialog ("ArrowHeads", NULL);
 }
 
 bool gcpArrowToolDlg::Apply ()
 {
-	gcpArrowTool *tool = (gcpArrowTool*) m_App->GetTool ("ReversibleArrow");
+	gcpArrowTool *tool = (gcpArrowTool*)
+			reinterpret_cast<gcpApplication*> (m_App)->GetTool ("ReversibleArrow");
 	if (tool)
 		tool->SetArrowType (gtk_toggle_button_get_active (full_btn)? gcpFullReversibleArrow: gcpReversibleArrow);
 	return true;
 }
 
-void gcpArrowToolDlg::OnDefault ()
-{
-	GConfClient *conf_client = gconf_client_get_default ();
-	gconf_client_set_bool (conf_client, "/apps/gchempaint/plugins/arrows/full-arrows-heads", gtk_toggle_button_get_active (full_btn), NULL);
-	g_object_unref (conf_client);
-}
-
 gcpArrowTool::gcpArrowTool(gcpApplication* App, unsigned ArrowType): gcpTool(App, ToolNames[ArrowType])
 {
 	points = gnome_canvas_points_new(2);
@@ -531,3 +526,98 @@ void gcpArrowTool::SetOptions ()
 	}
 	else gdk_window_raise (GTK_WIDGET(dlg->GetWindow())->window); 
 }
+
+static void on_full_toggled (GtkToggleButton *button, gcpArrowTool *tool)
+{
+	tool->SetArrowType (gtk_toggle_button_get_active (button)? gcpFullReversibleArrow: gcpReversibleArrow);
+}
+
+GtkWidget *gcpArrowTool::GetPropertyPage ()
+{
+	if (m_ArrowType != gcpReversibleArrow && m_ArrowType != gcpFullReversibleArrow)
+		return NULL;
+	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/arrowtool.glade", "heads-box", GETTEXT_PACKAGE);
+	GtkTable *table = GTK_TABLE (glade_xml_get_widget (xml, "heads-table"));
+	GnomeCanvas *canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
+	double width = (DefaultArrowLength + 2 * DefaultArrowPadding) * DefaultZoomFactor,
+		height = (DefaultArrowDist + DefaultBondWidth + 2 * (DefaultArrowHeadB + DefaultPadding)) * DefaultZoomFactor;
+	gtk_widget_set_size_request (GTK_WIDGET (canvas), (int) width, (int) height);
+	GnomeCanvasGroup *group = gnome_canvas_root (canvas);
+	GnomeCanvasPoints *points = gnome_canvas_points_new (2);
+	gnome_canvas_set_scroll_region (canvas, 0, 0, DefaultArrowLength, DefaultArrowDist + DefaultBondWidth + 2 * DefaultArrowHeadB);
+	points->coords[0] = (width - DefaultArrowLength * DefaultZoomFactor) / 2.;
+	points->coords[1] = points->coords[3] = (height - DefaultArrowDist * DefaultZoomFactor) / 2.;
+	points->coords[2] = (width + DefaultArrowLength * DefaultZoomFactor) / 2.;
+	gnome_canvas_item_new (
+						group,
+						gnome_canvas_line_ext_get_type (),
+						"points", points,
+						"fill_color", "black",
+						"width_units", DefaultBondWidth * DefaultZoomFactor,
+						"last_arrowhead", true,
+						"arrow_shape_a",DefaultArrowHeadA * DefaultZoomFactor,
+						"arrow_shape_b",DefaultArrowHeadB * DefaultZoomFactor,
+						"arrow_shape_c",DefaultArrowHeadC * DefaultZoomFactor,
+						"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
+						NULL);
+	points->coords[0] = (width + DefaultArrowLength * DefaultZoomFactor) / 2.;
+	points->coords[1] = points->coords[3] = (height + DefaultArrowDist * DefaultZoomFactor) / 2.;
+	points->coords[2] = (width - DefaultArrowLength * DefaultZoomFactor) / 2.;
+	gnome_canvas_item_new (
+						group,
+						gnome_canvas_line_ext_get_type (),
+						"points", points,
+						"fill_color", "black",
+						"width_units",DefaultBondWidth * DefaultZoomFactor,
+						"last_arrowhead", true,
+						"arrow_shape_a",DefaultArrowHeadA * DefaultZoomFactor,
+						"arrow_shape_b",DefaultArrowHeadB * DefaultZoomFactor,
+						"arrow_shape_c",DefaultArrowHeadC * DefaultZoomFactor,
+						"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
+						NULL);
+	gtk_widget_show (GTK_WIDGET (canvas));
+	gtk_table_attach (table, GTK_WIDGET (canvas), 1, 2, 0, 1, GTK_FILL, GTK_FILL, 10, 0);
+	canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
+	gtk_widget_set_size_request (GTK_WIDGET (canvas), (int) width, (int) height);
+	group = gnome_canvas_root (canvas);
+	gnome_canvas_set_scroll_region (canvas, 0, 0, DefaultArrowLength, DefaultArrowDist + DefaultBondWidth + 2 * DefaultArrowHeadB);
+	points->coords[0] = (width - DefaultArrowLength * DefaultZoomFactor) / 2.;
+	points->coords[1] = points->coords[3] = (height - DefaultArrowDist * DefaultZoomFactor) / 2.;
+	points->coords[2] = (width + DefaultArrowLength * DefaultZoomFactor) / 2.;
+	gnome_canvas_item_new (
+						group,
+						gnome_canvas_line_ext_get_type (),
+						"points", points,
+						"fill_color", "black",
+						"width_units", DefaultBondWidth * DefaultZoomFactor,
+						"last_arrowhead", true,
+						"arrow_shape_a",DefaultArrowHeadA * DefaultZoomFactor,
+						"arrow_shape_b",DefaultArrowHeadB * DefaultZoomFactor,
+						"arrow_shape_c",DefaultArrowHeadC * DefaultZoomFactor,
+						"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
+						NULL);
+	points->coords[0] = (width + DefaultArrowLength * DefaultZoomFactor) / 2.;
+	points->coords[1] = points->coords[3] = (height + DefaultArrowDist * DefaultZoomFactor) / 2.;
+	points->coords[2] = (width - DefaultArrowLength * DefaultZoomFactor) / 2.;
+	gnome_canvas_item_new (
+						group,
+						gnome_canvas_line_ext_get_type (),
+						"points", points,
+						"fill_color", "black",
+						"width_units",DefaultBondWidth * DefaultZoomFactor,
+						"last_arrowhead", true,
+						"arrow_shape_a",DefaultArrowHeadA * DefaultZoomFactor,
+						"arrow_shape_b",DefaultArrowHeadB * DefaultZoomFactor,
+						"arrow_shape_c",DefaultArrowHeadC * DefaultZoomFactor,
+						"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
+						NULL);
+	gtk_widget_show (GTK_WIDGET (canvas));
+	gtk_table_attach (table, GTK_WIDGET (canvas), 1, 2, 1, 2, GTK_FILL, GTK_FILL, 10, 0);
+	gnome_canvas_points_free (points);
+	GtkWidget *b = glade_xml_get_widget (xml, "full");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), m_ArrowType == gcpFullReversibleArrow);
+	g_signal_connect (G_OBJECT (b), "toggled", G_CALLBACK (on_full_toggled), this);
+	GtkWidget *w = glade_xml_get_widget (xml, "default");
+	g_signal_connect_swapped (w, "clicked", G_CALLBACK (on_default), b);
+	return glade_xml_get_widget (xml, "heads-box");
+}
