@@ -4,7 +4,7 @@
  * GChemPaint arrows plugin
  * retrosynthesis.cc
  *
- * Copyright (C) 2004-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2005 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,13 +28,13 @@
 #include "lib/document.h"
 #include "lib/widgetdata.h"
 #include "lib/view.h"
-#include <cmath>
+#include <math.h>
 
 TypeId RetrosynthesisType = NoType;
 
 gcpRetrosynthesis::gcpRetrosynthesis (): Object (RetrosynthesisType)
 {
-	SetId (const_cast<char*> ("rsy1"));
+	SetId ("rsy1");
 	Target = NULL;
 }
 
@@ -47,7 +47,7 @@ gcpRetrosynthesis::~gcpRetrosynthesis ()
 	gcpRetrosynthesisArrow *arrow;
 	gcpDocument *pDoc = reinterpret_cast<gcpDocument *> (GetDocument ());
 	gcpOperation *pOp = pDoc->GetCurrentOperation ();
-	while ((pObj = GetFirstChild (i))) {
+	while (pObj = GetFirstChild (i)) {
 		if (pObj->GetType () == RetrosynthesisArrowType) {
 			arrow = reinterpret_cast<gcpRetrosynthesisArrow *> (pObj);
 			arrow->SetStartStep (NULL);
@@ -287,8 +287,8 @@ int gcpRetrosynthesis::Validate (bool split)
 			return 2;
 		pObj = GetFirstChild (i);
 		while (pObj && (pObj->GetType () != RetrosynthesisStepType || 
-			(reinterpret_cast<gcpRetrosynthesisStep *> (pObj))->GetArrow () ||
-			pObj == Target))
+			(reinterpret_cast<gcpRetrosynthesisStep *> (pObj))->GetArrow ()) ||
+			pObj == Target)
 			pObj = GetNextChild (i);
 		if (reinterpret_cast<gcpRetrosynthesisStep *> (pObj)->Validate ()) {
 			gcpRetrosynthesis *rs = new gcpRetrosynthesis (GetParent (), 
@@ -442,7 +442,7 @@ bool gcpRetrosynthesis::OnSignal (SignalId Signal, Object *Child)
 
 gcpRetrosynthesis::gcpRetrosynthesis (Object* parent, gcpRetrosynthesisStep *step): Object (RetrosynthesisType)
 {
-	SetId (const_cast<char*> ("rsy1"));
+	SetId ("rsy1");
 	SetParent (parent);
 	Target = step;
 	AddChild (Target);
