@@ -4,7 +4,7 @@
  * GChemPaint bonds plugin
  * bondtool.cc
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2005 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -23,6 +23,7 @@
  */
 
 #include "gchempaint-config.h"
+#include <math.h>
 #include "bondtool.h"
 #include "lib/settings.h"
 #include "lib/mendeleiev.h"
@@ -31,7 +32,6 @@
 #include "lib/atom.h"
 #include "lib/bond.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
-#include <cmath>
 
 gcpBondTool::gcpBondTool (gcpApplication *App, string ToolId, unsigned nPoints): gcpTool (App, ToolId)
 {
@@ -46,6 +46,8 @@ gcpBondTool::~gcpBondTool ()
 
 bool gcpBondTool::OnClicked ()
 {
+	if (Element::GetMaxBonds (m_pApp->GetCurZ()) < 1)
+		return false;
 	int i;
 	m_pAtom = NULL;
 	m_pItem = NULL;
@@ -88,6 +90,11 @@ bool gcpBondTool::OnClicked ()
 				case 0:
 					break;
 				case 1: {
+					map<Atom*, Bond*>::iterator i;
+					gcpBond* bond = (gcpBond*) ((Atom*) m_pObject)->GetFirstBond (i);
+					m_dAngle = bond->GetAngle2D ((gcpAtom*) m_pObject);
+					m_dAngle += (m_nState & GDK_LOCK_MASK)?
+						DefaultBondAngle: -DefaultBondAngle;
 					break;
 				}
 				case 2: {
@@ -189,7 +196,7 @@ void gcpBondTool::OnDrag()
 		if (pItem) pObject = (Object*)g_object_get_data(G_OBJECT(pItem), "object");
 		double dAngle;
 		m_pAtom = NULL;
-		if (pObject)
+		if (MergeAtoms && pObject)
 		{
 			if (pObject->GetType() == BondType)
 			{
@@ -286,6 +293,7 @@ void gcpBondTool::OnRelease()
 		m_pOp->AddObject (m_pObjectGroup, 1);
 		pDoc->FinishOperation ();
 		m_pOp = NULL;
+		m_pObject->EmitSignal (OnChangedSignal);
 		return;
 	} else {
 		if (m_pOp)
@@ -298,7 +306,7 @@ void gcpBondTool::OnRelease()
 	Object* pObject = NULL;
 	if (pItem) pObject = (Object*)g_object_get_data(G_OBJECT(pItem), "object");
 	m_pAtom = NULL;
-	if (pObject)
+	if (MergeAtoms && pObject)
 	{
 		if (pObject->GetType() == BondType)
 		{
@@ -455,6 +463,43 @@ void gcpBondTool::SetType(gcpBond* pBond)
 	pBond->SetType(NormalBondType);
 }
 
+static void on_length_changed (GtkSpinButton *btn)
+{
+	DefaultBondLength = gtk_spin_button_get_value (btn);
+}
+
+static void on_angle_changed (GtkSpinButton *btn)
+{
+	DefaultBondAngle = gtk_spin_button_get_value (btn);
+}
+
+static void on_merge_toggled (GtkToggleButton *btn)
+{
+	MergeAtoms = gtk_toggle_button_get_active (btn);
+}
+
+GtkWidget *gcpBondTool::GetPropertyPage ()
+{
+	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/bond.glade", "bond", GETTEXT_PACKAGE);
+	m_LengthBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-length"));
+	g_signal_connect (m_LengthBtn, "value-changed", G_CALLBACK (on_length_changed), NULL);
+	m_AngleBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-angle"));
+	g_signal_connect (m_AngleBtn, "value-changed", G_CALLBACK (on_angle_changed), NULL);
+	m_MergeBtn = GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "merge"));
+	g_signal_connect (m_MergeBtn, "toggled", G_CALLBACK (on_merge_toggled), NULL);
+	return glade_xml_get_widget (xml, "bond");
+}
+
+extern bool new_ui;
+void gcpBondTool::Activate ()
+{
+	if (new_ui) {
+		gtk_spin_button_set_value (m_LengthBtn, DefaultBondLength);
+		gtk_spin_button_set_value (m_AngleBtn, DefaultBondAngle);
+		gtk_toggle_button_set_active (m_MergeBtn, MergeAtoms);
+	}
+}
+
 gcpUpBondTool::gcpUpBondTool(gcpApplication *App): gcpBondTool(App, "UpBond", 3)
 {
 }
@@ -620,6 +665,62 @@ void gcpDownBondTool::SetType(gcpBond* pBond)
 	pBond->SetType(DownBondType);
 }
 
+gcpForeBondTool::gcpForeBondTool(gcpApplication *App): gcpBondTool(App, "ForeBond", 4)
+{
+}
+
+gcpForeBondTool::~gcpForeBondTool()
+{
+}
+
+void gcpForeBondTool::Draw()
+{
+	double dx, dy, x1, y1, x2, y2;
+	x1 = sqrt(square(m_x1 - m_x0) + square(m_y1 - m_y0));
+	if (x1 == 0) return;
+	dx = (m_y0 - m_y1) / x1 * m_pData->StereoBondWidth / 2;
+	dy = (m_x1 - m_x0) / x1 * m_pData->StereoBondWidth / 2;
+	points->coords[0] = m_x0 + dx;
+	points->coords[1] = m_y0 + dy;
+	points->coords[2] = m_x1 + dx;
+	points->coords[3] = m_y1 + dy;
+	points->coords[4] = m_x1 - dx;
+	points->coords[5] = m_y1 - dy;
+	points->coords[6] = m_x0 - dx;
+	points->coords[7] = m_y0 - dy;
+	m_pItem = gnome_canvas_item_new(
+								m_pGroup,
+								gnome_canvas_polygon_get_type(),
+								"points", points,
+								"fill_color", AddColor,
+								NULL);
+	gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(m_pItem), &x1, &y1, &x2, &y2);
+	gnome_canvas_request_redraw(GNOME_CANVAS(m_pWidget), (int)x1, (int)y1, (int)x2, (int)y2);
+}
+
+void gcpForeBondTool::UpdateBond ()
+{
+	Draw();
+}
+
+void gcpForeBondTool::FinalizeBond ()
+{
+	if (m_bChanged)
+	{
+		gcpBond* pBond = (gcpBond*) m_pObject;
+		if (pBond->GetType () != ForeBondType) {
+			pBond->SetType (ForeBondType);
+			m_pView->Remove (m_pObject);
+			m_pView->AddObject (m_pObject);
+		}
+	}
+}
+
+void gcpForeBondTool::SetType (gcpBond* pBond)
+{
+	pBond->SetType (ForeBondType);
+}
+
 gcpSquiggleBondTool::gcpSquiggleBondTool(gcpApplication *App): gcpBondTool(App, "SquiggleBond", 4)
 {
 }
