@@ -557,7 +557,7 @@ void gcpCycleTool::OnRelease()
 		pItem = gnome_canvas_get_item_at(GNOME_CANVAS(m_pWidget), m_x, m_y);
 		if (pItem == (GnomeCanvasItem*)m_pBackground) pItem = NULL;
 		m_pObject = (pItem)? (Object*)g_object_get_data(G_OBJECT(pItem), "object"): NULL;
-		if (m_pObject)
+		if (MergeAtoms && m_pObject)
 		{
 			if (m_pObject->GetType() == BondType)
 			{
@@ -709,7 +709,7 @@ bool gcpCycleTool::CheckIfAllowed()
 		pItem = gnome_canvas_get_item_at(GNOME_CANVAS(m_pWidget), m_x, m_y);
 		if (pItem == (GnomeCanvasItem*)m_pBackground) pItem = NULL;
 		pObject = (pItem)? (Object*)g_object_get_data(G_OBJECT(pItem), "object"): NULL;
-		if (pObject)
+		if (MergeAtoms && pObject)
 		{
 			TypeId Id = pObject->GetType();
 			switch (Id)
@@ -738,6 +738,35 @@ bool gcpCycleTool::CheckIfAllowed()
 	return true;
 }
 
+static void on_length_changed (GtkSpinButton *btn)
+{
+	DefaultBondLength = gtk_spin_button_get_value (btn);
+}
+
+static void on_merge_toggled (GtkToggleButton *btn)
+{
+	MergeAtoms = gtk_toggle_button_get_active (btn);
+}
+
+GtkWidget *gcpCycleTool::GetPropertyPage ()
+{
+	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/cycle.glade", "cycle", GETTEXT_PACKAGE);
+	m_LengthBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-length"));
+	g_signal_connect (m_LengthBtn, "value-changed", G_CALLBACK (on_length_changed), NULL);
+	m_MergeBtn = GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "merge"));
+	g_signal_connect (m_MergeBtn, "toggled", G_CALLBACK (on_merge_toggled), NULL);
+	return glade_xml_get_widget (xml, "cycle");
+}
+
+extern bool new_ui;
+void gcpCycleTool::Activate ()
+{
+	if (new_ui) {
+		gtk_spin_button_set_value (m_LengthBtn, DefaultBondLength);
+		gtk_toggle_button_set_active (m_MergeBtn, MergeAtoms);
+	}
+}
+
 class gcpCNToolDlg: public Dialog
 {
 public:
@@ -776,15 +805,14 @@ static void on_size_changed (GtkSpinButton *button, gcpNCycleTool *tool)
 
 GtkWidget *gcpNCycleTool::GetPropertyPage ()
 {
-	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/cyclentool.glade", "size-box", GETTEXT_PACKAGE);
-	GtkWidget *w = glade_xml_get_widget (xml, "size-box");
-	GtkWidget *vbox = gtk_vbox_new (false, 0);
-	gtk_box_pack_start (GTK_BOX (vbox), w, false, false, 0);
-	gtk_widget_show (vbox);
-	w = glade_xml_get_widget (xml, "sizebtn");
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), m_size);
-	g_signal_connect (G_OBJECT (w), "value-changed", G_CALLBACK (on_size_changed), this);
-	return vbox;
+	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/cyclen.glade", "cycle", GETTEXT_PACKAGE);
+	m_LengthBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-length"));
+	g_signal_connect (m_LengthBtn, "value-changed", G_CALLBACK (on_length_changed), NULL);
+	m_MergeBtn = GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "merge"));
+	g_signal_connect (m_MergeBtn, "toggled", G_CALLBACK (on_merge_toggled), NULL);
+	m_SizeBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-length"));
+	g_signal_connect (m_SizeBtn, "value-changed", G_CALLBACK (on_size_changed), this);
+	return glade_xml_get_widget (xml, "cycle");
 }
 
 void gcpNCycleTool::SetOptions ()
