@@ -4,7 +4,7 @@
  * GChemPaint selection plugin
  * selectiontool.cc
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -30,19 +30,25 @@
 #include "lib/settings.h"
 #include "lib/document.h"
 #include "lib/application.h"
-#include <cmath>
+#include <math.h>
 #include <stdexcept>
 
 static void on_flip(GtkWidget *btn, gcpApplication* App)
 {
 	gcpSelectionTool *tool = (gcpSelectionTool*) App->GetTool ("Select");
-	tool->OnFlip (strcmp (gtk_widget_get_name (btn), "VertFlip"));
+	if (GTK_IS_WIDGET (btn))
+		tool->OnFlip (strcmp (gtk_widget_get_name (btn), "VertFlip"));
+	else
+		tool->OnFlip (strcmp (gtk_action_get_name (GTK_ACTION (btn)), "VertFlip"));
 }
 
 static void on_rotate(GtkWidget *btn, gcpApplication* App)
 {
 	gcpSelectionTool *tool = (gcpSelectionTool*) App->GetTool ("Select");
-	tool->Rotate (gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON (btn)));
+	if (GTK_IS_WIDGET (btn))
+		tool->Rotate (gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON (btn)));
+	else
+		tool->Rotate (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (btn)));
 }
 
 static void on_merge(GtkWidget *btn, gcpApplication* App)
@@ -57,10 +63,13 @@ gcpSelectionTool::gcpSelectionTool(gcpApplication *App): gcpTool(App, "Select")
 	m_pApp->SetCallback("rotate", (GCallback) on_rotate);
 	m_pApp->SetCallback("merge", (GCallback) on_merge);
 	m_bRotate = false;
+	m_UIManager = NULL;
 }
 
 gcpSelectionTool::~gcpSelectionTool()
 {
+	if (m_UIManager)
+		g_object_unref (m_UIManager);
 }
 
 bool gcpSelectionTool::OnClicked()
@@ -231,18 +240,18 @@ void gcpSelectionTool::OnRelease()
 		}
 	}
 	AddSelection(m_pData);
-	// If the selection is made of two molecules, activate the merge tool
-	m_pApp->ActivateToolItem ("Merge", ((m_pData->SelectedObjects.size () == 2) &&
-		(m_pData->SelectedObjects.front()->GetType() == MoleculeType) &&
-		(m_pData->SelectedObjects.back()->GetType() == MoleculeType)));
 }
 
 void gcpSelectionTool::Activate()
 {
-	m_pApp->ActivateToolItem ("HorizFlip", true);
-	m_pApp->ActivateToolItem ("VertFlip", true);
-	m_pApp->ActivateToolItem ("Rotate", true);
-	m_pApp->ActivateToolItem ("Merge", false);
+	if (m_UIManager)
+		gtk_widget_set_sensitive (m_MergeBtn, false);
+	else {
+		m_pApp->ActivateToolItem ("HorizFlip", true);
+		m_pApp->ActivateToolItem ("VertFlip", true);
+		m_pApp->ActivateToolItem ("Rotate", true);
+		m_pApp->ActivateToolItem ("Merge", false);
+	}
 	gcpDocument *pDoc = m_pApp->GetActiveDocument ();
 	if (pDoc) {
 		m_pView = m_pApp->GetActiveDocument ()->GetView ();
@@ -283,9 +292,19 @@ void gcpSelectionTool::AddSelection(gcpWidgetData* data)
 	}
 	SelectedWidgets.remove(m_pData);
 	SelectedWidgets.push_front(m_pData);
-	m_pData = d;
-	if (d) m_pView = d->View;
-	else m_pView = NULL;
+	if (d) {
+		m_pView = d->View;
+		m_pData = d;
+	}
+	// If the selection is made of two molecules, activate the merge tool
+	if (m_UIManager)
+		gtk_widget_set_sensitive (m_MergeBtn, ((m_pData->SelectedObjects.size () == 2) &&
+			(m_pData->SelectedObjects.front()->GetType() == MoleculeType) &&
+			(m_pData->SelectedObjects.back()->GetType() == MoleculeType)));
+	else
+		m_pApp->ActivateToolItem ("Merge", ((m_pData->SelectedObjects.size () == 2) &&
+			(m_pData->SelectedObjects.front()->GetType() == MoleculeType) &&
+			(m_pData->SelectedObjects.back()->GetType() == MoleculeType)));
 }
 
 void gcpSelectionTool::OnFlip (bool horizontal)
@@ -433,3 +452,83 @@ bool gcpSelectionTool::OnRightButtonClicked (GtkUIManager *UIManager)
 	}
 	return false;
 }
+
+/*	<toolitem name="HorizFlip"
+		_descr="Horizontal flip tool" pixtype="filename"
+		pixname="horiz24.png" _label="Horizontal flip"
+		_tip="Flip the selection horizontally" verb="flip"/>
+	<toolitem name="VertFlip"
+		_descr="Vertical flip tool" pixtype="filename"
+		pixname="vert24.png" _label="Vertical flip"
+		_tip="Flip the selection vertically" verb="flip"/>
+	<toolitem name="Rotate" type="toggle"
+		_descr="Rotation tool" pixtype="filename"
+		pixname="rotate24.png" _label="Rotate"
+		_tip="Rotate the selection" verb="rotate"/>
+	<toolitem name="Merge"
+		_descr="Merge molecules tool" pixtype="filename"
+		pixname="merge24.png" _label="Merge"
+		_tip="Merge two molecules" verb="merge"/>*/
+
+static GtkActionEntry entries[] = {
+	{ "HorizFlip", "gcp_Horiz", N_("Horizontal flip"), NULL,
+		N_("Flip the selection horizontally"), G_CALLBACK (on_flip) },
+	{ "VertFlip", "gcp_Vert", N_("Vertical flip"), NULL,
+		N_("Flip the selection vertically"), G_CALLBACK (on_flip) },
+	{ "Merge", "gcp_Merge", N_("Merge"), NULL,
+		N_("Merge two molecules"), G_CALLBACK (on_merge) }
+};
+
+static GtkToggleActionEntry toggles[] = {
+	  { "Rotate", "gcp_Rotate", N_("_Rotate"), NULL,
+		  N_("Rotate the selection"), G_CALLBACK (on_rotate), false }
+};
+
+static const char *ui_description =
+"<ui>"
+"  <toolbar name='Selection'>"
+"    <toolitem action='HorizFlip'/>"
+"    <toolitem action='VertFlip'/>"
+"    <toolitem action='Rotate'/>"
+"    <toolitem action='Merge'/>"
+"  </toolbar>"
+"</ui>";
+
+GtkWidget *gcpSelectionTool::GetPropertyPage ()
+{
+	GtkWidget *box, *w;
+	GtkActionGroup *action_group;
+//	GtkAccelGroup *accel_group;
+	GError *error;
+
+	box = gtk_vbox_new (FALSE, 0);
+	action_group = gtk_action_group_new ("SelectionToolActions");
+	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries), m_pApp);
+	gtk_action_group_add_toggle_actions (action_group, toggles, G_N_ELEMENTS (toggles), m_pApp);
+
+	m_UIManager = gtk_ui_manager_new ();
+	if (!gtk_ui_manager_add_ui_from_string (m_UIManager, ui_description, -1, &error))
+	  {
+		g_message ("building property page failed: %s", error->message);
+		g_error_free (error);
+		gtk_widget_destroy (box);
+		g_object_unref (m_UIManager);
+		m_UIManager = NULL;
+		return NULL;;
+	  }
+/*	g_object_connect (m_UIManager,
+		"signal::connect_proxy",    G_CALLBACK (on_connect_proxy), this,
+		"signal::disconnect_proxy", G_CALLBACK (on_disconnect_proxy), this,
+		NULL); */
+	gtk_ui_manager_insert_action_group (m_UIManager, action_group, 0);
+	w = gtk_ui_manager_get_widget (m_UIManager, "/Selection");
+	gtk_toolbar_set_style (GTK_TOOLBAR (w), GTK_TOOLBAR_ICONS);
+	gtk_toolbar_set_show_arrow (GTK_TOOLBAR (w), false);
+	gtk_toolbar_set_tooltips (GTK_TOOLBAR (w), true);
+	gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+	gtk_widget_show_all (box);
+	m_MergeBtn = gtk_ui_manager_get_widget (m_UIManager, "/Selection/Merge");
+	gtk_widget_set_sensitive (m_MergeBtn, false);
+	return box;
+}
