@@ -4,9 +4,7 @@
  * GChemPaint templates plugin
  * templatetool.h 
  *
- * Copyright (C) 2004
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -30,6 +28,7 @@
 #include "lib/tool.h"
 
 class gcpTemplate;
+class gcpDocument;
 
 class gcpTemplateTool: public gcpTool
 {
@@ -43,9 +42,14 @@ public:
 	virtual void SetOptions();
 	void SetTemplate (gcpTemplate* temp);
 	gcpTemplate* GetTemplate () {return m_Template;}
+	GtkWidget *GetPropertyPage ();
+	void OnChanged(GtkComboBox *combo);
+	void OnPreviewSize (GtkAllocation *allocation);
 
 private:
 	gcpTemplate* m_Template;
+	GtkWidget *m_DeleteBtn;
+	GtkNotebook *m_Book;
 };
 
 #endif //GCHEMPAINT_TEMPLATETOOL_H
