@@ -4,9 +4,7 @@
  * GChemPaint text plugin
  * texttool.cc 
  *
- * Copyright (C) 2002-2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -34,38 +32,53 @@
 #include <unistd.h>
 
 extern GtkTextTagTable* TextTagTable;
+extern bool new_ui;
 
-static bool on_toggled(GtkWidget *btn, gcpApplication* App)
+static bool on_toggled (GtkWidget *btn, gcpApplication* App)
 {
 	gcpTextTool* tool = (gcpTextTool*) App->GetTool ("Text");
-	return (tool)? tool->OnToggled(btn): false;
+	return (tool)? tool->OnToggled (btn): false;
 }
 
-static bool on_font(GtkWidget* w, gcpApplication* App)
+static bool on_font (GtkWidget* w, gcpApplication* App)
 {
 	gcpTextTool* tool = (gcpTextTool*) App->GetTool ("Text");
-	return (tool)? tool->OnFont(): false;
+	return (tool)? tool->OnFont (): false;
 }
 
-static void on_get_data(GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info, gcpTextTool* tool)
+static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info, gcpTextTool* tool)
 {
 	tool->OnGetData (clipboard, selection_data, info);
 }
 
-gcpTextTool::gcpTextTool(gcpApplication* App, string Id): gcpTool(App, Id)
+gcpTextTool::gcpTextTool (gcpApplication* App, string Id): gcpTool (App, Id)
 {
 	m_Active = NULL;
 	m_bUndo = true;
 	m_CurNode = m_InitNode = NULL;
-	if (!m_pApp->GetCallback("texttools"))
-	{
-		m_pApp->SetCallback("texttools", (GCallback) on_toggled);
-		m_pApp->SetCallback("font", (GCallback) on_font);
+	if (!m_pApp->GetCallback ("texttools")) {
+		m_pApp->SetCallback ("texttools", (GCallback) on_toggled);
+		m_pApp->SetCallback ("font", (GCallback) on_font);
 	}
+	m_FontDesc = NULL;
+	m_Style = DefaultFontStyle;
+	m_Weight = DefaultFontWeight;
+	m_Stretch = DefaultFontStretch;
+	m_Variant = DefaultFontVariant;
+	m_Size = DefaultFontSize;
+	m_Scalable = false; // force sizes list creation
 }
 
-gcpTextTool::~gcpTextTool()
+gcpTextTool::~gcpTextTool ()
 {
+	map<string, PangoFontFamily*>::iterator i, iend = m_Families.end ();
+	for (i = m_Families.begin (); i != iend; i++) {
+		g_object_unref ((*i).second);
+	}
+	map<string, PangoFontFace*>::iterator j, jend = m_Faces.end ();
+	for (j = m_Faces.begin (); j != jend; j++) {
+		g_object_unref ((*j).second);
+	}
 }
 
 bool gcpTextTool::OnClicked()
@@ -95,66 +108,93 @@ bool gcpTextTool::OnClicked()
 	return true;
 }
 
-bool gcpTextTool::OnEvent(GdkEvent* event)
+bool gcpTextTool::OnEvent (GdkEvent* event)
 {
-	if (m_Active)
-	{
-		if ((event->type == GDK_KEY_PRESS) || (event->type == GDK_KEY_RELEASE))
-		{
-			if (event->key.state & GDK_CONTROL_MASK)
-			{
-				switch(event->key.keyval)
-				{
-					case GDK_Right:
-					case GDK_Left:
-					case GDK_Up:
-					case GDK_Down:
-					case GDK_End:
-					case GDK_Home:
-					case GDK_Delete:
-					case GDK_KP_Delete:
-					case GDK_BackSpace:
-						break;
-					case GDK_a:
-						m_pView->OnSelectAll();
-						return true;
-					case GDK_z:
-						m_pView->GetDoc()->OnUndo();
-						return true;
-					case GDK_Z:
-						m_pView->GetDoc()->OnRedo();
-						return true;
-					case GDK_c:
-						CopySelection(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD));
-						return true;
-					case GDK_v:
-						PasteSelection(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD));
-						return true;
-					case GDK_x:
-						CutSelection(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD));
-						return true;
-					case GDK_i:
-						gtk_toggle_tool_button_set_active((GtkToggleToolButton*)m_pApp->GetToolItem("italic"),
-											!gtk_toggle_tool_button_get_active((GtkToggleToolButton*)m_pApp->GetToolItem("italic")));
-						return true;
-					case GDK_u:
-						gtk_toggle_tool_button_set_active((GtkToggleToolButton*)m_pApp->GetToolItem("underline"),
-											!gtk_toggle_tool_button_get_active((GtkToggleToolButton*)m_pApp->GetToolItem("underline")));
-						return true;
-					case GDK_b:
-						gtk_toggle_tool_button_set_active((GtkToggleToolButton*)m_pApp->GetToolItem("bold"),
-											!gtk_toggle_tool_button_get_active((GtkToggleToolButton*)m_pApp->GetToolItem("bold")));
-						return true;
-					case GDK_plus:
-						gtk_toggle_tool_button_set_active((GtkToggleToolButton*)m_pApp->GetToolItem("superscript"),
-											!gtk_toggle_tool_button_get_active((GtkToggleToolButton*)m_pApp->GetToolItem("superscript")));
-						return true;
-					case GDK_equal:
-						gtk_toggle_tool_button_set_active((GtkToggleToolButton*)m_pApp->GetToolItem("subscript"),
-											!gtk_toggle_tool_button_get_active((GtkToggleToolButton*)m_pApp->GetToolItem("subscript")));
-						return true;
-					default:
-						return true;
+	if (m_Active) {
+		if ((event->type == GDK_KEY_PRESS) || (event->type == GDK_KEY_RELEASE)) {
+			if (event->key.state & GDK_CONTROL_MASK) {
+				switch (event->key.keyval) {
+				case GDK_Right:
+				case GDK_Left:
+				case GDK_Up:
+				case GDK_Down:
+				case GDK_End:
+				case GDK_Home:
+				case GDK_Delete:
+				case GDK_KP_Delete:
+				case GDK_BackSpace:
+					break;
+				case GDK_a:
+					m_pView->OnSelectAll ();
+					return true;
+				case GDK_z:
+					m_pView->GetDoc()->OnUndo ();
+					return true;
+				case GDK_Z:
+					m_pView->GetDoc()->OnRedo ();
+					return true;
+				case GDK_c:
+					CopySelection (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD));
+					return true;
+				case GDK_v:
+					PasteSelection (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD));
+					return true;
+				case GDK_x:
+					CutSelection (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD));
+					return true;
+				case GDK_i:
+					if (new_ui) {
+						m_Style = (m_Style == PANGO_STYLE_NORMAL)? PANGO_STYLE_ITALIC: PANGO_STYLE_NORMAL;
+						SelectBestFontFace ();
+					} else
+						gtk_toggle_tool_button_set_active ((GtkToggleToolButton*) m_pApp->GetToolItem ("italic"),
+										!gtk_toggle_tool_button_get_active ((GtkToggleToolButton*) m_pApp->GetToolItem ("italic")));
+					BuildAttributeList ();
+					return true;
+				case GDK_u:
+					if (new_ui) {
+					} else
+						gtk_toggle_tool_button_set_active ((GtkToggleToolButton*)m_pApp->GetToolItem ("underline"),
+										!gtk_toggle_tool_button_get_active ((GtkToggleToolButton*) m_pApp->GetToolItem ("underline")));
+					BuildAttributeList ();
+					return true;
+				case GDK_b:
+					if (new_ui) {
+						m_Weight = (m_Weight == PANGO_WEIGHT_NORMAL)? PANGO_WEIGHT_BOLD: PANGO_WEIGHT_NORMAL;
+						SelectBestFontFace ();
+					} else
+						gtk_toggle_tool_button_set_active ((GtkToggleToolButton*) m_pApp->GetToolItem ("bold"),
+										!gtk_toggle_tool_button_get_active ((GtkToggleToolButton*) m_pApp->GetToolItem ("bold")));
+					BuildAttributeList ();
+					return true;
+				case GDK_plus:
+				case GDK_dead_circumflex:
+					if (new_ui) {
+					} else
+						gtk_toggle_tool_button_set_active ((GtkToggleToolButton*) m_pApp->GetToolItem ("superscript"),
+										!gtk_toggle_tool_button_get_active ((GtkToggleToolButton*) m_pApp->GetToolItem ("superscript")));
+					BuildAttributeList ();
+					return true;
+				case GDK_equal:
+				case GDK_underscore:
+					if (new_ui) {
+					} else
+						gtk_toggle_tool_button_set_active ((GtkToggleToolButton*) m_pApp->GetToolItem ("subscript"),
+										!gtk_toggle_tool_button_get_active ((GtkToggleToolButton*) m_pApp->GetToolItem ("subscript")));
+					BuildAttributeList ();
+					return true;
+				case GDK_space:
+					if (new_ui) {
+					} else {
+						if (gtk_toggle_tool_button_get_active ((GtkToggleToolButton*) m_pApp->GetToolItem ("superscript")))
+							gtk_toggle_tool_button_set_active ((GtkToggleToolButton*) m_pApp->GetToolItem ("superscript"), false);
+						else if (gtk_toggle_tool_button_get_active ((GtkToggleToolButton*) m_pApp->GetToolItem ("subscript")))
+							gtk_toggle_tool_button_set_active ((GtkToggleToolButton*) m_pApp->GetToolItem ("subscript"), false);
+					}
+					BuildAttributeList ();
+					return true;
+				default:
+					return false;
 				}
 			}
 			if (!g_utf8_validate(((GdkEventKey*)event)->string, -1, NULL))
@@ -618,3 +658,266 @@ void gcpTextTool::OnGetData(GtkClipboard *clipboard, GtkSelectionData *selection
 	cleared = false;
 	if (clipboard == gtk_clipboard_get (GDK_SELECTION_CLIPBOARD)) m_pApp->ActivateMenu("Paste", true);
 }
+
+void gcpTextTool::BuildAttributeList ()
+{
+	m_Dirty = false;
+}
+
+static void on_select_family (GtkTreeSelection *selection, gcpTextTool *tool)
+{
+	tool->OnSelectFamily (selection);
+}
+
+static void on_select_face (GtkTreeSelection *selection, gcpTextTool *tool)
+{
+	tool->OnSelectFace (selection);
+}
+
+static void on_select_size (GtkTreeSelection *selection, gcpTextTool *tool)
+{
+	int size;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gtk_tree_selection_get_selected (selection, &model, &iter);
+	gtk_tree_model_get (model, &iter, 0, &size, -1);
+	tool->OnSelectSize (size * PANGO_SCALE);
+}
+
+static void on_size_activate (GtkEntry *entry, gcpTextTool *tool)
+{
+}
+
+static void on_size_focus_out (GtkEntry *entry, GdkEventFocus *event, gcpTextTool *tool)
+{
+}
+
+GtkWidget *gcpTextTool::GetPropertyPage ()
+{
+	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/fontsel.glade", "fontsel", GETTEXT_PACKAGE);
+	GtkWidget *w = glade_xml_get_widget (xml, "underline");
+	gtk_combo_box_set_active (GTK_COMBO_BOX (w), 0); // FIXME: set the current underline style
+	PangoFontFamily **families;
+	int i, nb;
+	// Initialize faces list
+	m_FaceList = gtk_list_store_new (1, G_TYPE_STRING);
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (m_FaceList), 0, GTK_SORT_ASCENDING);
+	m_FacesTree = (GtkTreeView *) glade_xml_get_widget (xml, "style");
+	gtk_tree_view_set_model (m_FacesTree, GTK_TREE_MODEL (m_FaceList));
+	GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
+	GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes (NULL, renderer, "text", 0, NULL);
+	gtk_tree_view_append_column (m_FacesTree, column);
+	GtkTreeSelection *selection = gtk_tree_view_get_selection (m_FacesTree);
+	m_FaceSel = selection;
+	m_FaceSignal = g_signal_connect (m_FaceSel, "changed", G_CALLBACK (on_select_face), this);
+	// Initialize sizes list
+	m_SizeList = gtk_list_store_new (1, G_TYPE_INT);
+	GtkTreeView *tree = (GtkTreeView *) glade_xml_get_widget (xml, "size-list");
+	gtk_tree_view_set_model (tree, GTK_TREE_MODEL (m_SizeList));
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes (NULL, renderer, "text", 0, NULL);
+	gtk_tree_view_append_column (tree, column);
+	selection = gtk_tree_view_get_selection (tree);
+	m_SizeSel = selection;
+	m_SizeSignal = g_signal_connect (m_SizeSel, "changed", G_CALLBACK (on_select_size), this);
+	// Size entry
+	m_SizeEntry = (GtkEntry*) glade_xml_get_widget (xml, "size-entry");
+	g_signal_connect (m_SizeEntry, "activate", G_CALLBACK (on_size_activate), this);
+	g_signal_connect_after (m_SizeEntry, "focus_out_event", G_CALLBACK (on_size_focus_out), this);
+	PangoContext *pc = gtk_widget_get_pango_context (w);
+	pango_context_list_families (pc, &families, &nb);
+	m_FamilyList = gtk_list_store_new (1, G_TYPE_STRING);
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (m_FamilyList), 0, GTK_SORT_ASCENDING);
+	tree = (GtkTreeView *) glade_xml_get_widget (xml, "family");
+	gtk_tree_view_set_model (tree, GTK_TREE_MODEL (m_FamilyList));
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes (NULL, renderer, "text", 0, NULL);
+	gtk_tree_view_append_column (tree, column);
+	GtkTreeIter iter, selected;
+	GtkTreePath *path = NULL;
+	string name;
+	gtk_tree_model_get_iter_first   (GTK_TREE_MODEL (m_FamilyList), &selected);
+	for (i = 0; i < nb; i++) {
+		name = pango_font_family_get_name (families[i]);
+		m_Families[name] = (PangoFontFamily*) g_object_ref (families[i]);
+		gtk_list_store_append (m_FamilyList, &iter);
+		gtk_list_store_set (m_FamilyList, &iter,
+				  0, name.c_str (),
+				  -1);
+		if (name == DefaultTextFontFamily)
+			selected = iter;
+//			path = gtk_tree_model_get_path (GTK_TREE_MODEL (m_FamilyList), &iter);
+	}
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (m_FamilyList), &selected);
+	selection = gtk_tree_view_get_selection (tree);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
+	g_signal_connect (G_OBJECT (selection), "changed", G_CALLBACK (on_select_family), this);
+	// Select default font
+	if (path) {
+		gtk_tree_selection_select_path (selection, path);
+		gtk_tree_view_scroll_to_cell (tree, path, column, FALSE, 0., 0.);
+		gtk_tree_path_free (path);
+	} else {
+		//TODO: choose a font when default is not available
+	}
+
+	return glade_xml_get_widget (xml, "fontsel");
+}
+
+void gcpTextTool::OnSelectFamily (GtkTreeSelection *selection)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter, selected;
+	char const *name;
+	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+		return;
+	gtk_tree_model_get (model, &iter, 0, &name, -1);
+	PangoFontFamily *family = m_Families[name];
+	PangoFontFace **faces;
+	int i, besti, nb;
+	g_signal_handler_block (m_FaceSel, m_FaceSignal);
+	pango_font_family_list_faces (family, &faces, &nb);
+	gtk_list_store_clear (m_FaceList);
+	map<string, PangoFontFace*>::iterator j, jend = m_Faces.end ();
+	for (j = m_Faces.begin (); j != jend; j++) {
+		g_object_unref ((*j).second);
+	}
+	m_Faces.clear ();
+	PangoFontDescription *desc;
+	int distance, best;
+	PangoStyle Style;
+	PangoWeight Weight;
+	PangoVariant Variant;
+	PangoStretch Stretch;
+
+	best = 32000; // This should be enough
+	for (i = 0; i < nb; i++) {
+		name = pango_font_face_get_face_name (faces[i]);
+		desc = pango_font_face_describe (faces[i]);
+		m_Faces[name] = (PangoFontFace*) g_object_ref (faces[i]);
+		gtk_list_store_append (m_FaceList, &iter);
+		gtk_list_store_set (m_FaceList, &iter,
+				  0, name,
+				  -1);
+		// Try to select the best available face
+		Style = pango_font_description_get_style (desc);
+		Weight = pango_font_description_get_weight (desc);
+		Variant = pango_font_description_get_variant (desc);
+		Stretch = pango_font_description_get_stretch (desc);
+		distance = abs (Weight - m_Weight)
+						+ abs ((Style? Style + 2: 0) - (m_Style? m_Style + 2: 0)) * 1000
+						+ abs (Variant - m_Variant) * 10 + abs (Stretch - m_Stretch);
+		if (distance < best) {
+			best = distance;
+			selected = iter;
+			besti = i;
+		}
+		// TODO: write this code
+		pango_font_description_free (desc);
+	}
+	g_signal_handler_unblock (m_FaceSel, m_FaceSignal);
+	GtkTreePath *path = gtk_tree_model_get_path (GTK_TREE_MODEL (m_FaceList), &selected);
+	if (path) {
+		gtk_tree_selection_select_path (GTK_TREE_SELECTION (m_FaceSel), path);
+		gtk_tree_path_free (path);
+	} else {
+		//TODO: choose a face when default is not available
+	}
+}
+
+/* These are what we use as the standard font sizes, for the size list.
+ */
+static const guint16 font_sizes[] = {
+  8, 9, 10, 11, 12, 13, 14, 16, 18, 20, 22, 24, 26, 28,
+  32, 36, 40, 48, 56, 64, 72
+};
+
+void gcpTextTool::OnSelectFace (GtkTreeSelection *selection)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	char const *name;
+	g_signal_handler_block (m_SizeSel, m_SizeSignal);
+	m_Dirty = true;
+	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+		return;
+	gtk_tree_model_get (model, &iter, 0, &name, -1);
+	PangoFontFace *face = m_Faces[name];
+	PangoFontDescription *desc = pango_font_face_describe (face);
+	m_Style = pango_font_description_get_style (desc);
+	m_Weight = pango_font_description_get_weight (desc);
+	m_Variant = pango_font_description_get_variant (desc);
+	m_Stretch = pango_font_description_get_stretch (desc);
+	pango_font_description_free (desc);
+	// Populate sizes
+	int i, nb, *sizes;
+	pango_font_face_list_sizes (face, &sizes, &nb);
+	if (nb == 0) {
+		if (!m_Scalable) {
+			m_Scalable = true;
+			gtk_list_store_clear (m_SizeList);
+			for (i = 0; i < (int) G_N_ELEMENTS (font_sizes); i++) {
+				gtk_list_store_append (m_SizeList, &iter);
+				gtk_list_store_set (m_SizeList, &iter,
+						  0, font_sizes[i],
+						  -1);
+			}
+		}
+	} else {
+		m_Scalable = false;
+		gtk_list_store_clear (m_SizeList);
+	}
+	g_signal_handler_unblock (m_SizeSel, m_SizeSignal);
+	if (m_Dirty)
+		BuildAttributeList ();
+}
+
+void gcpTextTool::OnSelectSize (int size)
+{
+	BuildAttributeList ();
+}
+
+void gcpTextTool::SelectBestFontFace ()
+{
+	PangoFontDescription *desc;
+	int distance, best;
+	PangoStyle Style;
+	PangoWeight Weight;
+	PangoVariant Variant;
+	PangoStretch Stretch;
+	map <string, PangoFontFace*>::iterator i, iend = m_Faces.end ();
+	char const *name = NULL, *buf;
+	GtkTreeIter iter;
+
+	best = 32000; // This should be enough
+	for (i = m_Faces.begin (); i != iend; i++) {
+		desc = pango_font_face_describe ((*i).second);
+		// Try to select the best available face
+		Style = pango_font_description_get_style (desc);
+		Weight = pango_font_description_get_weight (desc);
+		Variant = pango_font_description_get_variant (desc);
+		Stretch = pango_font_description_get_stretch (desc);
+		distance = abs (Weight - m_Weight)
+						+ abs ((Style? Style + 2: 0) - (m_Style? m_Style + 2: 0)) * 1000
+						+ abs (Variant - m_Variant) * 10 + abs (Stretch - m_Stretch);
+		if (distance < best) {
+			best = distance;
+			name = (*i).first.c_str ();
+		}
+		pango_font_description_free (desc);
+	}
+	// select the found face
+	gtk_tree_model_get_iter_first (GTK_TREE_MODEL (m_FaceList), &iter);
+	do {
+		gtk_tree_model_get (GTK_TREE_MODEL (m_FaceList), &iter, 0, &buf, -1);
+		if (!strcmp (name, buf)) {
+			m_Dirty = true;
+			GtkTreePath *path = gtk_tree_model_get_path (GTK_TREE_MODEL (m_FaceList), &iter);
+			gtk_tree_view_set_cursor (m_FacesTree, path, NULL, FALSE);
+			gtk_tree_path_free (path);
+			if (m_Dirty)
+				OnSelectFace ((GtkTreeSelection*) m_FaceSel);
+			break;
+		}
+	} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (m_FaceList), &iter));
+}
