@@ -4,9 +4,7 @@
  * GChemPaint
  * standaloneapp.cc
  *
- * Copyright (C) 2004-2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2005 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,9 +24,10 @@
 
 #include "gchempaint-config.h"
 #include "standaloneapp.h"
+#include "standalonewin.h"
 #include "lib/about.h"
-#include "lib/filechooser.h"
 #include "lib/mendeleiev.h"
+#include <gcu/filechooser.h>
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-job.h>
 #include <libgnomeprintui/gnome-print-dialog.h>
@@ -461,109 +460,116 @@ static const char *ui_web_description =
 #undef PACKAGE
 #define PACKAGE "gchempaint-unstable" 
 
+extern bool new_ui;
+
 gcpStandaloneApp::gcpStandaloneApp(): gcpApplication()
 {
-	GtkWidget *vbox;
-	GtkWidget *bar;
-	GtkWidget *item;
-	GtkActionGroup *action_group;
-	GtkAccelGroup *accel_group;
-	GError *error;
-
-	m_NumWindow = 1;
-	m_NumDoc = 0;
-	m_MessageId = 0;
-	m_Window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
-	gtk_window_set_title (m_Window, _("GChemPaint"));
-	gtk_window_set_icon_name (m_Window, "gchempaint");
-	g_signal_connect(G_OBJECT(m_Window), "destroy", G_CALLBACK(on_quit), this);
-	g_signal_connect(G_OBJECT(m_Window), "focus_in_event", G_CALLBACK(on_focus), this);
-	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (m_Window), vbox);
-	action_group = gtk_action_group_new ("MenuActions");
-	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries), this);
-	gtk_action_group_add_toggle_actions (action_group, toggle_entries, G_N_ELEMENTS (toggle_entries), this);
-
-	m_UIManager = gtk_ui_manager_new ();
-	g_object_connect (m_UIManager,
-		"signal::connect_proxy",    G_CALLBACK (on_connect_proxy), this,
-		"signal::disconnect_proxy", G_CALLBACK (on_disconnect_proxy), this,
-		NULL);
-	gtk_ui_manager_insert_action_group (m_UIManager, action_group, 0);
-
-	accel_group = gtk_ui_manager_get_accel_group (m_UIManager);
-	gtk_window_add_accel_group (GTK_WINDOW (m_Window), accel_group);
+	if (new_ui) {
+	} else {
+		GtkWidget *vbox;
+		GtkWidget *bar;
+		GtkWidget *item;
+		GtkActionGroup *action_group;
+		GtkAccelGroup *accel_group;
+		GError *error;
 	
-	error = NULL;
-	if (!gtk_ui_manager_add_ui_from_string (m_UIManager, ui_description, -1, &error))
-	  {
-		g_message ("building menus failed: %s", error->message);
-		g_error_free (error);
-		exit (EXIT_FAILURE);
-	  }
-	if (WebBrowser[0])
-		gtk_ui_manager_add_ui_from_string (m_UIManager, ui_web_description, -1, &error);
-	if (MailAgent[0])
-		gtk_ui_manager_add_ui_from_string (m_UIManager, ui_mail_description, -1, &error);
-
-	bar = gtk_ui_manager_get_widget (m_UIManager, "/MainMenu");
-	gtk_box_pack_start (GTK_BOX (vbox), bar, false, false, 0);
-	m_Dock = bonobo_dock_new ();
-	gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (m_Dock), true, true, 0);
-	bar = gtk_ui_manager_get_widget (m_UIManager, "/MainToolbar");
-	gtk_toolbar_set_style(GTK_TOOLBAR(bar), GTK_TOOLBAR_ICONS);
-	gtk_toolbar_set_show_arrow(GTK_TOOLBAR(bar), false);
-	gtk_toolbar_set_tooltips(GTK_TOOLBAR(bar), true);
-	item = bonobo_dock_item_new (_("Main"), BONOBO_DOCK_ITEM_BEH_NORMAL);
-	gtk_container_add (GTK_CONTAINER (item), bar);
-	bonobo_dock_add_item (BONOBO_DOCK (m_Dock),BONOBO_DOCK_ITEM (item), BONOBO_DOCK_TOP, 0, 0, 0, true);
-
-	m_Book = GTK_NOTEBOOK(gtk_notebook_new());
-	gtk_notebook_set_tab_pos(m_Book, GTK_POS_TOP);
-	bonobo_dock_set_client_area (BONOBO_DOCK (m_Dock), GTK_WIDGET (m_Book));
-
-	m_Bar = gtk_statusbar_new ();
-	m_statusId = gtk_statusbar_get_context_id (GTK_STATUSBAR (m_Bar), "status");
-	gtk_statusbar_push (GTK_STATUSBAR (m_Bar), m_statusId, _("Ready"));
-	gtk_box_pack_start (GTK_BOX (vbox), m_Bar, false, false, 0);
-	SetMenu("Undo", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Undo"));
-	SetMenu("Redo", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Redo"));
-	SetMenu("Copy", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Copy"));
-	SetMenu("Cut", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Cut"));
-	SetMenu("Paste", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Paste"));
-	SetMenu("Erase", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Delete"));
-	SetMenu("Image", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/FileMenu/SaveAsImage"));
-	SetMenu("Save", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/FileMenu/Save"));
-	SetToolItem("Save", gtk_ui_manager_get_widget (m_UIManager, "/MainToolbar/Save"));
-	ActivateMenu("Undo", false);
-	ActivateMenu("Redo", false);
-	ActivateMenu("Copy", false);
-	ActivateMenu("Cut", false);
-	ActivateMenu("Paste", false);
-	ActivateMenu("Erase", false);
-	ActivateMenu("Image", false);
-
-	Callbacks["tools"] = (GCallback) on_tool_clicked;
-
-	if (m_pNode && !strcmp(bonobo_ui_node_get_name(m_pNode), "Root")) {
-		BonoboUINode* Child = bonobo_ui_node_children (m_pNode);
-		AppendUI (Child);
-	}
+		m_NumWindow = 1;
+		m_NumDoc = 0;
+		m_MessageId = 0;
+		m_Window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
+		gtk_window_set_title (m_Window, _("GChemPaint"));
+		gtk_window_set_icon_name (m_Window, "gchempaint");
+		g_signal_connect(G_OBJECT(m_Window), "destroy", G_CALLBACK(on_quit), this);
+		g_signal_connect(G_OBJECT(m_Window), "focus_in_event", G_CALLBACK(on_focus), this);
+		vbox = gtk_vbox_new (FALSE, 0);
+		gtk_container_add (GTK_CONTAINER (m_Window), vbox);
+		action_group = gtk_action_group_new ("MenuActions");
+		gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
+		gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries), this);
+		gtk_action_group_add_toggle_actions (action_group, toggle_entries, G_N_ELEMENTS (toggle_entries), this);
+	
+		m_UIManager = gtk_ui_manager_new ();
+		g_object_connect (m_UIManager,
+			"signal::connect_proxy",    G_CALLBACK (on_connect_proxy), this,
+			"signal::disconnect_proxy", G_CALLBACK (on_disconnect_proxy), this,
+			NULL);
+		gtk_ui_manager_insert_action_group (m_UIManager, action_group, 0);
+	
+		accel_group = gtk_ui_manager_get_accel_group (m_UIManager);
+		gtk_window_add_accel_group (GTK_WINDOW (m_Window), accel_group);
 		
-	g_signal_connect(GTK_OBJECT(m_Window), "key_press_event", (GCallback)on_key_press, this);
-	g_signal_connect(GTK_OBJECT(m_Window), "key_release_event", (GCallback)on_key_release, this);
-
-	Callbacks["tools"] = (GCallback) on_tool_clicked;
-
-	SetMenu("Mendeleiev", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/ViewMenu/Mendeleiev"));
-	gcpMendeleievDlg::SetElement (this, m_CurZ);
+		error = NULL;
+		if (!gtk_ui_manager_add_ui_from_string (m_UIManager, ui_description, -1, &error))
+		  {
+			g_message ("building menus failed: %s", error->message);
+			g_error_free (error);
+			exit (EXIT_FAILURE);
+		  }
+		if (WebBrowser[0])
+			gtk_ui_manager_add_ui_from_string (m_UIManager, ui_web_description, -1, &error);
+		if (MailAgent[0])
+			gtk_ui_manager_add_ui_from_string (m_UIManager, ui_mail_description, -1, &error);
+	
+		bar = gtk_ui_manager_get_widget (m_UIManager, "/MainMenu");
+		gtk_box_pack_start (GTK_BOX (vbox), bar, false, false, 0);
+		m_Dock = bonobo_dock_new ();
+		gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (m_Dock), true, true, 0);
+		bar = gtk_ui_manager_get_widget (m_UIManager, "/MainToolbar");
+		gtk_toolbar_set_style(GTK_TOOLBAR(bar), GTK_TOOLBAR_ICONS);
+		gtk_toolbar_set_show_arrow(GTK_TOOLBAR(bar), false);
+		gtk_toolbar_set_tooltips(GTK_TOOLBAR(bar), true);
+		item = bonobo_dock_item_new (_("Main"), BONOBO_DOCK_ITEM_BEH_NORMAL);
+		gtk_container_add (GTK_CONTAINER (item), bar);
+		bonobo_dock_add_item (BONOBO_DOCK (m_Dock),BONOBO_DOCK_ITEM (item), BONOBO_DOCK_TOP, 0, 0, 0, true);
+	
+		m_Book = GTK_NOTEBOOK(gtk_notebook_new());
+		gtk_notebook_set_tab_pos(m_Book, GTK_POS_TOP);
+		bonobo_dock_set_client_area (BONOBO_DOCK (m_Dock), GTK_WIDGET (m_Book));
+	
+		m_Bar = gtk_statusbar_new ();
+		m_statusId = gtk_statusbar_get_context_id (GTK_STATUSBAR (m_Bar), "status");
+		gtk_statusbar_push (GTK_STATUSBAR (m_Bar), m_statusId, _("Ready"));
+		gtk_box_pack_start (GTK_BOX (vbox), m_Bar, false, false, 0);
+		SetMenu("Undo", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Undo"));
+		SetMenu("Redo", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Redo"));
+		SetMenu("Copy", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Copy"));
+		SetMenu("Cut", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Cut"));
+		SetMenu("Paste", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Paste"));
+		SetMenu("Erase", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Delete"));
+		SetMenu("Image", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/FileMenu/SaveAsImage"));
+		SetMenu("Save", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/FileMenu/Save"));
+		SetToolItem("Save", gtk_ui_manager_get_widget (m_UIManager, "/MainToolbar/Save"));
+		ActivateMenu("Undo", false);
+		ActivateMenu("Redo", false);
+		ActivateMenu("Copy", false);
+		ActivateMenu("Cut", false);
+		ActivateMenu("Paste", false);
+		ActivateMenu("Erase", false);
+		ActivateMenu("Image", false);
+	
+		Callbacks["tools"] = (GCallback) on_tool_clicked;
+	
+		if (m_pNode && !strcmp(bonobo_ui_node_get_name(m_pNode), "Root")) {
+			BonoboUINode* Child = bonobo_ui_node_children (m_pNode);
+			AppendUI (Child);
+		}
+			
+		g_signal_connect(GTK_OBJECT(m_Window), "key_press_event", (GCallback)on_key_press, this);
+		g_signal_connect(GTK_OBJECT(m_Window), "key_release_event", (GCallback)on_key_release, this);
+	
+		Callbacks["tools"] = (GCallback) on_tool_clicked;
+	
+		SetMenu("Mendeleiev", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/ViewMenu/Mendeleiev"));
+		gcpMendeleievDlg::SetElement (this, m_CurZ);
+	}
 }
 
 gcpStandaloneApp::~gcpStandaloneApp()
 {
 	while (!m_Docs.empty())
 		delete *m_Docs.begin();
+	if (!new_ui)
+		g_object_unref (m_UIManager);
 }
 
 void gcpStandaloneApp::OnToolClicked (GtkWidget* widget)
@@ -601,45 +607,51 @@ void gcpStandaloneApp::OnRightClick(GtkWidget* widget)
 void gcpStandaloneApp::OnFileNew()
 {
 	gchar tmp[32];
-	if (m_pActiveDoc && !m_pActiveDoc->GetView()->PrepareUnselect()) return;
-	g_snprintf(tmp, sizeof(tmp), _("Untitled %d"), m_NumWindow++);
-	GtkWidget* w = CreateView();
-	if (w)
-	{
-		GtkWidget *label = gtk_label_new(tmp);
-		g_snprintf(tmp, sizeof(tmp), "win%d", m_NumDoc++);
-		GtkAction *action = gtk_action_new (tmp, gtk_label_get_text (GTK_LABEL (label)), NULL, NULL);
-		GtkActionGroup *group = gtk_action_group_new (tmp);
-		GError* error;
-		gtk_action_group_add_action (group, action);
-		gtk_ui_manager_insert_action_group (m_UIManager, group, 0);
-		gchar* buf = g_strconcat ("<ui><menubar name='MainMenu'><menu action='WindowsMenu'><menuitem action='",
-		tmp, "'/></menu></menubar></ui>", NULL);
-		error = NULL;
-		long id = gtk_ui_manager_add_ui_from_string (m_UIManager, buf, -1, NULL);
-		g_free (buf);
-		buf = g_strconcat ("/MainMenu/WindowsMenu/", tmp, NULL);
-		GtkWidget* item = gtk_ui_manager_get_widget (m_UIManager, buf);
-		g_free (buf);
-		g_signal_connect(G_OBJECT(item), "activate", (GtkSignalFunc)on_select, w); 
-		g_signal_connect(G_OBJECT(w), "expose_event", (GtkSignalFunc)on_page_select, NULL);
-		g_object_set_data(G_OBJECT(w), "menu-id", (void*) id);
-		g_object_set_data(G_OBJECT(w), "label", label);
-		GtkScrolledWindow* scroll = (GtkScrolledWindow*)gtk_scrolled_window_new(NULL, NULL);
-		gtk_scrolled_window_set_policy(scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-		gtk_scrolled_window_set_shadow_type (scroll, GTK_SHADOW_IN);
-		gtk_scrolled_window_add_with_viewport(scroll, w);
-		gtk_widget_set_size_request(GTK_WIDGET(scroll), 408, 308);
-		gtk_widget_show(GTK_WIDGET(scroll));
-		gtk_notebook_append_page(m_Book, GTK_WIDGET(scroll), label);
-		gtk_notebook_set_current_page(m_Book, g_list_length(m_Book->children) - 1);
-		m_pActiveDoc = (gcpDocument*) g_object_get_data(G_OBJECT(w), "doc");
+	if (m_pActiveDoc && !m_pActiveDoc->GetView ()->PrepareUnselect ())
+		return;
+	g_snprintf (tmp, sizeof (tmp), _("Untitled %d"), m_NumWindow++);
+	if (new_ui) {
+		gcpWindow *win = new gcpStandaloneWindow (this);
+		win->SetTitle (tmp);
+		m_Docs.push_back (win->GetDocument ());
+	} else {
+		GtkWidget* w = CreateView ();
+		if (w) {
+			GtkWidget *label = gtk_label_new(tmp);
+			g_snprintf(tmp, sizeof(tmp), "win%d", m_NumDoc++);
+			GtkAction *action = gtk_action_new (tmp, gtk_label_get_text (GTK_LABEL (label)), NULL, NULL);
+			GtkActionGroup *group = gtk_action_group_new (tmp);
+			GError* error;
+			gtk_action_group_add_action (group, action);
+			gtk_ui_manager_insert_action_group (m_UIManager, group, 0);
+			gchar* buf = g_strconcat ("<ui><menubar name='MainMenu'><menu action='WindowsMenu'><menuitem action='",
+			tmp, "'/></menu></menubar></ui>", NULL);
+			error = NULL;
+			long id = gtk_ui_manager_add_ui_from_string (m_UIManager, buf, -1, NULL);
+			g_free (buf);
+			buf = g_strconcat ("/MainMenu/WindowsMenu/", tmp, NULL);
+			GtkWidget* item = gtk_ui_manager_get_widget (m_UIManager, buf);
+			g_free (buf);
+			g_signal_connect(G_OBJECT(item), "activate", (GtkSignalFunc)on_select, w); 
+			g_signal_connect(G_OBJECT(w), "expose_event", (GtkSignalFunc)on_page_select, NULL);
+			g_object_set_data(G_OBJECT(w), "menu-id", (void*) id);
+			g_object_set_data(G_OBJECT(w), "label", label);
+			GtkScrolledWindow* scroll = (GtkScrolledWindow*)gtk_scrolled_window_new(NULL, NULL);
+			gtk_scrolled_window_set_policy(scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+			gtk_scrolled_window_set_shadow_type (scroll, GTK_SHADOW_IN);
+			gtk_scrolled_window_add_with_viewport(scroll, w);
+			gtk_widget_set_size_request(GTK_WIDGET(scroll), 408, 308);
+			gtk_widget_show(GTK_WIDGET(scroll));
+			gtk_notebook_append_page(m_Book, GTK_WIDGET(scroll), label);
+			gtk_notebook_set_current_page(m_Book, g_list_length(m_Book->children) - 1);
+			m_pActiveDoc = (gcpDocument*) g_object_get_data(G_OBJECT(w), "doc");
+		}
 	}
 }
 
 void gcpStandaloneApp::OnFileOpen()
 {
-	new gcpFileChooser (this, false);
+	FileChooser (this, false, GetSupportedMimeTypes ());
 }
 
 void  gcpStandaloneApp::AddDocument(gcpDocument* pDoc)
@@ -688,9 +700,14 @@ void  gcpStandaloneApp::AddDocument(gcpDocument* pDoc)
 
 void gcpStandaloneApp::RemoveDocument(gcpDocument* pDoc)
 {
-	GtkWidget *w = pDoc->GetWidget ();
-	gtk_ui_manager_remove_ui (m_UIManager, (long) g_object_get_data (G_OBJECT (w), "menu-id"));
+	if (!new_ui) {
+		GtkWidget *w = pDoc->GetWidget ();
+		gtk_ui_manager_remove_ui (m_UIManager, (long) g_object_get_data (G_OBJECT (w), "menu-id"));
+	}
 	m_Docs.remove (pDoc);
+	if (new_ui && m_Docs.size () == 0) {
+		gtk_main_quit ();
+	}
 }
 
 GtkWidget* gcpStandaloneApp::CreateView()
@@ -724,9 +741,11 @@ void gcpStandaloneApp::ActivateToolItem(const string& itemname, bool activate)
 	if (w) gtk_widget_set_sensitive(w, activate);
 }
 
-void gcpStandaloneApp::ClearStatus()
+void gcpStandaloneApp::ClearStatus ()
 {
-	if (m_MessageId) {
+	if (new_ui)
+		m_pActiveWin->ClearStatus ();
+	else if (m_MessageId) {
 		gtk_statusbar_pop (GTK_STATUSBAR (m_Bar), m_statusId);
 		m_MessageId = 0;
 	}
@@ -734,13 +753,21 @@ void gcpStandaloneApp::ClearStatus()
 
 void gcpStandaloneApp::SetStatusText(const char* text)
 {
-	if (m_MessageId) gtk_statusbar_pop (GTK_STATUSBAR (m_Bar), m_statusId);
-	m_MessageId = gtk_statusbar_push (GTK_STATUSBAR (m_Bar), m_statusId, text);
+	if (new_ui)
+		m_pActiveWin->SetStatusText (text);
+	else {
+		if (m_MessageId)
+			gtk_statusbar_pop (GTK_STATUSBAR (m_Bar), m_statusId);
+		m_MessageId = gtk_statusbar_push (GTK_STATUSBAR (m_Bar), m_statusId, text);
+	}
 }
 
 GtkWindow* gcpStandaloneApp::GetWindow()
 {
-	return m_Window;
+	if (new_ui)
+		return m_pActiveWin->GetWindow ();
+	else
+		return m_Window;
 }
 
 void gcpStandaloneApp::AppendUI (BonoboUINode* Node)
@@ -941,7 +968,8 @@ void gcpStandaloneApp::OnPrint()
 	pc = gnome_print_job_get_context (gpj);
 	gnome_print_beginpage(pc, (const guchar*)"");
 	gdouble width, height;
-	gnome_print_config_get_page_size(config, &width, &height);
+	gnome_print_config_get_double (config, (guchar const*) GNOME_PRINT_KEY_PAPER_WIDTH, &width);
+	gnome_print_config_get_double (config, (guchar const*) GNOME_PRINT_KEY_PAPER_HEIGHT, &height);
 	m_pActiveDoc->Print(pc, width, height);
 	gnome_print_showpage(pc);
 	g_object_unref(pc);
