@@ -35,14 +35,14 @@ create_app (void)
 					"a container for GChemPaint BonoboControl test");
 
 	/* get a widget, containing the control */
-	control = bonobo_widget_new_control ("OAFIID:gchempaint_control", BONOBO_OBJREF (uic));
+	control = bonobo_widget_new_control ("OAFIID:gchempaint_unstable_control", BONOBO_OBJREF (uic));
 	if (!control) 
 		g_error ("Can't create control\n");
 	bonobo_window_set_contents(bonobo_win,control);
 
 	gtk_widget_show_all (GTK_WIDGET(bonobo_win));
 	
-	uri = "file://"SRCDIR;
+	uri = "file://"SRCDIR"/tests";
 	bu = bonobo_widget_get_objref (BONOBO_WIDGET(control));
 	bs = Bonobo_Unknown_queryInterface(bu,"IDL:Bonobo/PersistStream:1.0", &ev);
 	storage = bonobo_get_object (uri, "IDL:Bonobo/Storage:1.0", &ev);
@@ -71,7 +71,7 @@ main (int argc, char** argv)
 	 * We can't make any CORBA calls unless we're in the main
 	 * loop.  So we delay creating the container here.
 	 */
-	gtk_idle_add ((GtkFunction) create_app, NULL);
+	g_idle_add ((GtkFunction) create_app, NULL);
 	bonobo_main ();
 
 	return 0;
